@@ -1,8 +1,12 @@
-#include <zypp/zyppng/base/EventDispatcher>
-#include <zypp/zyppng/media/network/downloader.h>
-#include <zypp/zyppng/media/network/networkrequesterror.h>
-#include <zypp/zyppng/media/network/networkrequestdispatcher.h>
-#include <zypp/zyppng/media/network/request.h>
+#include <zypp-core/zyppng/base/EventLoop>
+#include <zypp-core/zyppng/base/EventDispatcher>
+#include <zypp-curl/ng/network/Downloader>
+#include <zypp-curl/ng/network/DownloadSpec>
+#include <zypp-curl/ng/network/NetworkRequestError>
+#include <zypp-curl/ng/network/NetworkRequestDispatcher>
+#include <zypp-curl/ng/network/Request>
+#include <zypp-media/auth/CredentialManager>
+#include <zypp/Digest.h>
 #include <zypp/TmpPath.h>
 #include <zypp/PathInfo.h>
 #include <zypp/ZConfig.h>
@@ -10,20 +14,22 @@
 #include <fstream>
 #include <random>
 #include "WebServer.h"
+#include "TestTools.h"
 
 #include <boost/test/unit_test.hpp>
 #include <boost/test/data/test_case.hpp>
 
 #define BOOST_TEST_REQ_SUCCESS(REQ) \
   do { \
-      BOOST_REQUIRE_MESSAGE( REQ->state() == zyppng::Download::Success, zypp::str::Format(" %1% != zyppng::Download::Success (%2%)") % REQ->state() % REQ->errorString() ); \
+      BOOST_REQUIRE_MESSAGE( REQ->state() == zyppng::Download::Finished && !REQ->hasError(), zypp::str::Format(" %1% != zyppng::Download::Success (%2%)") % REQ->state() % REQ->errorString() ); \
       BOOST_REQUIRE_EQUAL( REQ->lastRequestError().type(), zyppng::NetworkRequestError::NoError ); \
       BOOST_REQUIRE( REQ->errorString().empty() ); \
   } while(false)
 
 #define BOOST_TEST_REQ_FAILED(REQ) \
   do { \
-      BOOST_REQUIRE_EQUAL( REQ->state(), zyppng::Download::Failed ); \
+      BOOST_REQUIRE_EQUAL( REQ->state(), zyppng::Download::Finished ); \
+      BOOST_REQUIRE( REQ->hasError() ); \
       BOOST_REQUIRE_NE( REQ->lastRequestError().type(), zyppng::NetworkRequestError::NoError ); \
       BOOST_REQUIRE( !REQ->errorString().empty() ); \
   } while(false)
@@ -32,28 +38,12 @@ namespace bdata = boost::unit_test::data;
 
 bool withSSL[] = {true, false};
 
-//read all contents of a file into a string)
-std::string readFile ( const zypp::Pathname &file )
-{
-  if ( ! zypp::PathInfo( file ).isFile() ) {
-    return std::string();
-  }
-  std::ifstream istr( file.asString().c_str() );
-  if ( ! istr ) {
-    return std::string();
-  }
-
-  std::string str((std::istreambuf_iterator<char>(istr)),
-    std::istreambuf_iterator<char>());
-  return str;
-}
-
-
 BOOST_DATA_TEST_CASE( dltest_basic, bdata::make( withSSL ), withSSL)
 {
-  auto ev = zyppng::EventDispatcher::createMain();
+  auto ev = zyppng::EventLoop::create();
 
-  zyppng::Downloader downloader;
+  zyppng::Downloader::Ptr downloader = std::make_shared<zyppng::Downloader>();
+  downloader->setCredManagerOptions( zypp::media::CredManagerOptions( zypp::ZConfig::instance().repoManagerRoot()) );
 
   //make sure the data here is big enough to cross the threshold of 256 bytes so we get a progress signal emitted and not only the alive signal.
   std::string dummyContent = "This is just some dummy content,\nto test downloading and signals.\n"
@@ -80,8 +70,8 @@ BOOST_DATA_TEST_CASE( dltest_basic, bdata::make( withSSL ), withSSL)
   std::vector<zyppng::Download::State> allStates;
 
 
-  zyppng::Download::Ptr dl = downloader.downloadFile(  weburl, targetFile.path(), dummyContent.length() );
-  dl->settings() = set;
+  zyppng::Download::Ptr dl = downloader->downloadFile(  zyppng::DownloadSpec(weburl, targetFile.path(), dummyContent.length()) );
+  dl->spec().setTransferSettings( set );
 
   dl->sigFinished().connect([&]( zyppng::Download & ){
     gotFinished = true;
@@ -115,19 +105,25 @@ BOOST_DATA_TEST_CASE( dltest_basic, bdata::make( withSSL ), withSSL)
   BOOST_REQUIRE( gotAlive );
   BOOST_REQUIRE_EQUAL( totalDL, dummyContent.length() );
   BOOST_REQUIRE_EQUAL( lastProgress, dummyContent.length() );
-  BOOST_REQUIRE ( allStates == std::vector<zyppng::Download::State>({zyppng::Download::Initializing,zyppng::Download::Running, zyppng::Download::Success}) );
+  BOOST_REQUIRE ( allStates == std::vector<zyppng::Download::State>({zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::Finished}) );
 }
 
-
 struct MirrorSet
 {
   std::string name; //<dataset name, used only in debug output if the test fails
-  std::string handlerPath; //< the webhandler path used to query the resource
+  std::string filename; //< the name of the file we want to query, the test will base all other handlers and filenames on this
+  zypp::ByteCount dlTotal;
   std::vector< std::pair<int, std::string> > mirrors; //all mirrors injected into the metalink file
   int expectedFileDownloads; //< how many downloads are direct file downloads
   int expectedHandlerDownloads; //< how many started downloads are handler requests
   std::vector<zyppng::Download::State> expectedStates;
   bool expectSuccess; //< should the download work out?
+  zypp::ByteCount chunkSize = zypp::ByteCount( 256, zypp::ByteCount::K );
+
+  //zck data
+  zypp::CheckSum  headerChecksum; //< ZChunk header checksum
+  zypp::ByteCount headerSize;     //< ZChunk header size
+  std::string     deltaFilePath;  //< ZChunk delta file
 
   std::ostream & operator<<( std::ostream & str ) const {
     str << "MirrorSet{ " << name << " }";
@@ -151,38 +147,71 @@ std::vector< MirrorSet > generateMirr ()
   //all mirrors good:
   res.push_back( MirrorSet() );
   res.back().name = "All good mirrors";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads  = 1;
   res.back().expectedFileDownloads  = 9;
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
+  for ( int i = 100 ; i >= 10; i -= 10 )
+    res.back().mirrors.push_back( std::make_pair( i, "/test.txt") );
+
+  //all mirrors good big chunk
+  res.push_back( MirrorSet() );
+  res.back().name = "All good mirrors, 1024 chunk size";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
+  res.back().expectSuccess = true;
+  res.back().expectedHandlerDownloads  = 1;
+  res.back().expectedFileDownloads  = 3;
+  res.back().chunkSize = zypp::ByteCount( 1024, zypp::ByteCount::K );
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
   for ( int i = 100 ; i >= 10; i -= 10 )
     res.back().mirrors.push_back( std::make_pair( i, "/test.txt") );
 
   //no mirrors:
   res.push_back( MirrorSet() );
   res.back().name = "Empty mirrors";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads  = 10;
   res.back().expectedFileDownloads  = 0;
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
 
   //only broken mirrors:
   res.push_back( MirrorSet() );
   res.back().name = "All broken mirrors";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads  = 2; //has to fall back to url handler download
   res.back().expectedFileDownloads  = 10; //should try all mirrors and fail
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Running, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::DlSimple, zyppng::Download::Finished};
   for ( int i = 100 ; i >= 10; i -= 10 )
     res.back().mirrors.push_back( std::make_pair( i, "/doesnotexist.txt") );
 
+  //only broken mirrors:
+  res.push_back( MirrorSet() );
+  res.back().name = "All broken mirrors by params";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
+  res.back().expectSuccess = true;
+  res.back().expectedHandlerDownloads  = 2; //has to fall back to url handler download
+  res.back().expectedFileDownloads  = 0; // Setting up the mirrors will fail before even starting a download, so we should only see requests to the handler directly
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::DlSimple, zyppng::Download::Finished};
+  for ( int i = 100 ; i >= 10; i -= 10 )
+    res.back().mirrors.push_back( std::make_pair( i, "/test.txt?auth=foobar") );
+
   //some broken mirrors:
   res.push_back( MirrorSet() );
   res.back().name = "Some broken mirrors less URLs than blocks";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads = 1;
   res.back().expectedFileDownloads = 9 + 3; // 3 should fail due to broken mirrors
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
   for ( int i = 10 ; i >= 5; i-- ) {
     if ( i % 2 ) {
       res.back().mirrors.push_back( std::make_pair( i*10, "/doesnotexist.txt") );
@@ -194,13 +223,15 @@ std::vector< MirrorSet > generateMirr ()
   //some broken mirrors with more URLs than blocks:
   res.push_back( MirrorSet() );
   res.back().name = "Some broken mirrors more URLs than blocks";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads = 1;
   //its not really possible to know how many times the downloads will fail, there are
   //5 broken mirrors in the set, but if a working mirror is done before the last broken
   //URL is picked from the dataset not all broken URLs will be used
   res.back().expectedFileDownloads  = -1;
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
   for ( int i = 10 ; i >= 1; i-- ) {
     if ( i % 2 ) {
       res.back().mirrors.push_back( std::make_pair( i*10, "/doesnotexist.txt") );
@@ -212,13 +243,15 @@ std::vector< MirrorSet > generateMirr ()
   //mirrors where some return a invalid block
   res.push_back( MirrorSet() );
   res.back().name = "Some mirrors return broken blocks";
+  res.back().filename = "test.txt";
+  res.back().dlTotal = 2148018;
   res.back().expectSuccess = true;
   res.back().expectedHandlerDownloads  = 1;
   //its not really possible to know how many times the downloads will fail, there are
   //5 broken mirrors in the set, but if a working mirror is done before the last broken
   //URL is picked from the dataset not all broken URLs will be used
   res.back().expectedFileDownloads  = -1;
-  res.back().expectedStates = {zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success};
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished};
   for ( int i = 10 ; i >= 1; i-- ) {
     if ( i % 2 ) {
       res.back().mirrors.push_back( std::make_pair( i*10, "/handler/random") );
@@ -226,6 +259,25 @@ std::vector< MirrorSet > generateMirr ()
       res.back().mirrors.push_back( std::make_pair( i*10, "/test.txt") );
     }
   }
+
+#if ENABLE_ZCHUNK_COMPRESSION
+  //all mirrors good with zck:
+  res.push_back( MirrorSet() );
+  res.back().name = "All good mirrors with zck";
+  res.back().filename = "primary.xml.zck";
+  res.back().dlTotal = 274638;
+  res.back().expectSuccess = true;
+  res.back().expectedHandlerDownloads  = 2; // query if metalink avail + dl metalink
+  res.back().expectedFileDownloads  = 6; // Zck Head + 5 Chunks ( chunk size is at least 4K but is likely a few bytes bigger )
+  res.back().expectedStates = { zyppng::Download::InitialState, zyppng::Download::DetectMetaLink, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlZChunkHead, zyppng::Download::DlZChunk, zyppng::Download::Finished};
+  res.back().headerChecksum = zypp::CheckSum( zypp::Digest::sha256(), "90a1a1b99ba3b6c8ae9f14b0c8b8c43141c69ec3388bfa3b9915fbeea03926b7");
+  res.back().headerSize     = 11717;
+  res.back().deltaFilePath  = "primary-deltatemplate.xml.zck";
+  res.back().chunkSize      = zypp::ByteCount( 4, zypp::ByteCount::K );
+  for ( int i = 100 ; i >= 10; i -= 10 )
+    res.back().mirrors.push_back( std::make_pair( i, "/primary.xml.zck") );
+#endif
+
   return res;
 }
 
@@ -233,7 +285,7 @@ std::vector< MirrorSet > generateMirr ()
 //create one URL line for a metalink template file
 std::string makeUrl ( int pref, const zyppng::Url &url )
 {
-  return ( zypp::str::Format( "<url preference=\"%1%\" location=\"de\" type=\"%2%\">%3%</url>" ) % pref % url.getScheme() % url );
+  return ( zypp::str::Format( "<url preference=\"%1%\" location=\"de\" type=\"%2%\">%3%</url>" ) % pref % url.getScheme() % url.asCompleteString() );
 };
 
 
@@ -251,23 +303,23 @@ static bool requestWantsMetaLink ( const WebServer::Request &req )
 
 //creates a request handler for the Mock WebServer that returns the metalink data
 //specified in \a data if the request has the metalink accept handler
-WebServer::RequestHandler makeMetaFileHandler ( const std::string *data )
+WebServer::RequestHandler makeMetaFileHandler ( const std::string &fName, const std::string *data )
 {
-  return [ data ]( WebServer::Request &req ){
+  return [ data, fName ]( WebServer::Request &req ){
     if ( requestWantsMetaLink( req ) ) {
       req.rout << WebServer::makeResponseString( "200", { "Content-Type: application/metalink+xml; charset=utf-8\r\n" }, *data );
       return;
     }
-    req.rout << "Location: /test.txt\r\n\r\n";
+    req.rout << "Location: /"<<fName<<"\r\n\r\n";
     return;
   };
 };
 
 //creates a request handler for the Mock WebServer that returns a junk block of
 //data for a range request, otherwise relocates the request
-WebServer::RequestHandler makeJunkBlockHandler ( )
+WebServer::RequestHandler makeJunkBlockHandler ( const std::string &fName )
 {
-  return [ ]( WebServer::Request &req ){
+  return [ fName ]( WebServer::Request &req ){
     auto it = req.params.find( "HTTP_RANGE" );
     if ( it != req.params.end() && zypp::str::startsWith( it->second, "bytes=" ) ) {
         //bytes=786432-1048575
@@ -282,17 +334,19 @@ WebServer::RequestHandler makeJunkBlockHandler ( )
 
         if ( start != -1 && end != -1 ) {
           std::string block;
-          for ( off_t curr = 0; curr < ( end - start); curr++ ) {
+          for ( off_t curr = start; curr <= end; curr++ ) {
             block += 'a';
           }
           req.rout << "Status: 206 Partial Content\r\n"
                    << "Accept-Ranges: bytes\r\n"
-                   << "Content-Length: "<<( end - start)<<"\r\n\r\n"
+                   << "Content-Length: "<< block.length() <<"\r\n"
+                   << "Content-Range: bytes "<<start<<"-"<<end<<"/"<<block.length()<<"\r\n"
+                   <<"\r\n"
                    << block;
           return;
         }
     }
-    req.rout << "Location: /test.txt\r\n\r\n";
+    req.rout << "Location: /"<<fName<<"\r\n\r\n";
     return;
   };
 };
@@ -301,36 +355,42 @@ int maxConcurrentDLs[] = { 1, 2, 4, 8, 10, 15 };
 
 BOOST_DATA_TEST_CASE( test1, bdata::make( generateMirr() ) * bdata::make( withSSL ) * bdata::make( maxConcurrentDLs )  , elem, withSSL, maxDLs )
 {
-  //each URL in the metalink file has a preference , a schema and of course the URL, we need to adapt those to our test setup
-  //so we generate the file on the fly from a template in the test data
-  std::string metaTempl = readFile ( zypp::Pathname(TESTS_SRC_DIR)/"/zyppng/data/downloader/test.txt.meta" );
+
+  zypp::Pathname testRoot = zypp::Pathname(TESTS_SRC_DIR)/"zyppng/data/downloader";
+
+  // std::cout << "Starting with test " << elem.name << std::endl;
+
+  // each URL in the metalink file has a preference , a schema and of course the URL, we need to adapt those to our test setup
+  // so we generate the file on the fly from a template in the test data
+  std::string metaTempl = TestTools::readFile ( testRoot/( zypp::str::Format("%1%.meta") % elem.filename ).str() );
   BOOST_REQUIRE( !metaTempl.empty() );
 
-  auto ev = zyppng::EventDispatcher::createMain();
+  auto ev = zyppng::EventLoop::create();
 
-  WebServer web((zypp::Pathname(TESTS_SRC_DIR)/"/zyppng/data/downloader").c_str(), 10001, withSSL );
+  WebServer web( testRoot.c_str(), 10001, withSSL );
   BOOST_REQUIRE( web.start() );
 
   zypp::filesystem::TmpFile targetFile;
-  zyppng::Downloader downloader;
-  downloader.requestDispatcher()->setMaximumConcurrentConnections( maxDLs );
+  std::shared_ptr<zyppng::Downloader> downloader = std::make_shared<zyppng::Downloader>();
+  downloader->setCredManagerOptions( zypp::media::CredManagerOptions( zypp::ZConfig::instance().repoManagerRoot()) );
+  downloader->requestDispatcher()->setMaximumConcurrentConnections( maxDLs );
 
   //first metalink download, generate a fully valid one
   zyppng::Url weburl (web.url());
-  weburl.setPathName("/handler/test.txt");
+  weburl.setPathName( zypp::str::Format("/handler/%1%") % elem.filename );
 
   std::string urls;
   if ( elem.mirrors.size() ) {
     for ( const auto &mirr : elem.mirrors ) {
-      zyppng::Url mirrUrl (web.url());
-      mirrUrl.setPathName( mirr.second );
+      zyppng::Url mirrUrl ( web.url().asCompleteString() + mirr.second );
+      // mirrUrl.setPathName( mirr.second );
       urls += makeUrl( mirr.first, mirrUrl ) + "\n";
     }
   }
 
   std::string metaFile = zypp::str::Format( metaTempl ) % urls;
-  web.addRequestHandler("test.txt", makeMetaFileHandler( &metaFile ) );
-  web.addRequestHandler("random", makeJunkBlockHandler( ) );
+  web.addRequestHandler( elem.filename, makeMetaFileHandler(  elem.filename, &metaFile ) );
+  web.addRequestHandler("random", makeJunkBlockHandler( elem.filename ) );
 
   int expectedDownloads = elem.expectedHandlerDownloads + elem.expectedFileDownloads;
   int startedDownloads = 0;
@@ -346,8 +406,13 @@ BOOST_DATA_TEST_CASE( test1, bdata::make( generateMirr() ) * bdata::make( withSS
   int countHandlerReq = 0; //the requests made to the handler slot
   int countFileReq = 0;    //the requests made to the file directly, e.g. a mirror read from the metalink file
 
-  auto dl = downloader.downloadFile( weburl, targetFile );
-  dl->settings() = web.transferSettings();
+  auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile) );
+  dl->spec().setTransferSettings( web.transferSettings() )
+    .setPreferredChunkSize( elem.chunkSize )
+    .setHeaderSize( elem.headerSize )
+    .setHeaderChecksum( elem.headerChecksum )
+    .setDeltaFile( elem.deltaFilePath.empty() ? zypp::Pathname() : testRoot/elem.deltaFilePath )
+    .setExpectedFileSize( elem.dlTotal );
 
   dl->dispatcher().sigDownloadStarted().connect( [&]( zyppng::NetworkRequestDispatcher &, zyppng::NetworkRequest &req){
     startedDownloads++;
@@ -380,7 +445,7 @@ BOOST_DATA_TEST_CASE( test1, bdata::make( generateMirr() ) * bdata::make( withSS
   });
 
   dl->sigStateChanged().connect([&]( zyppng::Download &, zyppng::Download::State state ){
-    if ( state == zyppng::Download::RunningMulti )
+    if ( state == zyppng::Download::DlMetalink || state == zyppng::Download::DlZChunk )
       gotMultiDLState = true;
   });
 
@@ -398,14 +463,16 @@ BOOST_DATA_TEST_CASE( test1, bdata::make( generateMirr() ) * bdata::make( withSS
   else
     BOOST_TEST_REQ_FAILED ( dl );
 
-  if ( elem.expectedHandlerDownloads > -1 && elem.expectedFileDownloads > -1 )
+  if ( elem.expectedHandlerDownloads > -1 && elem.expectedFileDownloads > -1 ) {
     BOOST_REQUIRE_EQUAL( startedDownloads, expectedDownloads );
+  }
 
   BOOST_REQUIRE_EQUAL( startedDownloads, finishedDownloads );
   BOOST_REQUIRE( gotAlive );
   BOOST_REQUIRE( gotProgress );
   BOOST_REQUIRE( gotMultiDLState );
-  BOOST_REQUIRE_EQUAL( lastProgress, 2148018 );
+  if ( elem.dlTotal > 0 )
+    BOOST_REQUIRE_EQUAL( lastProgress, elem.dlTotal );
   BOOST_REQUIRE_EQUAL( lastProgress, gotTotal );
 
   if ( elem.expectedHandlerDownloads > -1 )
@@ -469,9 +536,10 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
   zypp::filesystem::TmpDir repoManagerRoot;
   zypp::ZConfig::instance().setRepoManagerRoot( repoManagerRoot.path() );
 
-  auto ev = zyppng::EventDispatcher::createMain();
+  auto ev = zyppng::EventLoop::create();
 
-  zyppng::Downloader downloader;
+  std::shared_ptr<zyppng::Downloader> downloader = std::make_shared<zyppng::Downloader>();
+  downloader->setCredManagerOptions( zypp::media::CredManagerOptions( zypp::ZConfig::instance().repoManagerRoot()) );
 
   WebServer web((zypp::Pathname(TESTS_SRC_DIR)/"/zyppng/data/downloader").c_str(), 10001, withSSL );
   BOOST_REQUIRE( web.start() );
@@ -485,8 +553,8 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
   web.addRequestHandler( "quit", [ &ev ]( WebServer::Request & ){ ev->quit();} );
 
   {
-    auto dl = downloader.downloadFile( weburl, targetFile.path() );
-    dl->settings() = set;
+    auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile.path()) );
+    dl->spec().setTransferSettings(set);
 
     dl->sigFinished( ).connect([ &ev ]( zyppng::Download & ){
       ev->quit();
@@ -499,8 +567,8 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
   }
 
   {
-    auto dl = downloader.downloadFile( weburl, targetFile.path() );
-    dl->settings() = set;
+    auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile.path()) );
+    dl->spec().setTransferSettings(set);
 
     int gotAuthRequest = 0;
 
@@ -526,8 +594,8 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
   {
     int gotAuthRequest = 0;
     std::vector<zyppng::Download::State> allStates;
-    auto dl = downloader.downloadFile( weburl, targetFile.path() );
-    dl->settings() = set;
+    auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile.path()) );
+    dl->spec().setTransferSettings(set);
 
     dl->sigFinished( ).connect([ &ev ]( zyppng::Download & ){
       ev->quit();
@@ -548,14 +616,14 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
     ev->run();
     BOOST_TEST_REQ_SUCCESS( dl );
     BOOST_REQUIRE_EQUAL( gotAuthRequest, 1 );
-    BOOST_REQUIRE ( allStates == std::vector<zyppng::Download::State>({zyppng::Download::Initializing, zyppng::Download::RunningMulti, zyppng::Download::Success}) );
+    BOOST_REQUIRE ( allStates == std::vector<zyppng::Download::State>({ zyppng::Download::InitialState, zyppng::Download::DlMetaLinkInfo, zyppng::Download::PrepareMulti, zyppng::Download::DlMetalink, zyppng::Download::Finished}) );
   }
 
   {
     //the creds should be in the credential manager now, we should not need to specify them again in the slot
     bool gotAuthRequest = false;
-    auto dl = downloader.downloadFile( weburl, targetFile.path() );
-    dl->settings() = set;
+    auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile.path()) );
+    dl->spec().setTransferSettings(set);
 
     dl->sigFinished( ).connect([ &ev ]( zyppng::Download & ){
       ev->quit();
@@ -571,3 +639,67 @@ BOOST_DATA_TEST_CASE( dltest_auth, bdata::make( withSSL ), withSSL )
     BOOST_REQUIRE( !gotAuthRequest );
   }
 }
+
+/**
+ * Test for bsc#1174011 auth=basic ignored in some cases
+ *
+ * If the URL specifes ?auth=basic libzypp should proactively send credentials we have available in the cred store
+ */
+BOOST_DATA_TEST_CASE( dltest_auth_basic, bdata::make( withSSL ), withSSL )
+{
+  //don't write or read creds from real settings dir
+  zypp::filesystem::TmpDir repoManagerRoot;
+  zypp::ZConfig::instance().setRepoManagerRoot( repoManagerRoot.path() );
+
+  auto ev = zyppng::EventLoop::create();
+
+  std::shared_ptr<zyppng::Downloader> downloader = std::make_shared<zyppng::Downloader>();
+  downloader->setCredManagerOptions( zypp::media::CredManagerOptions( zypp::ZConfig::instance().repoManagerRoot()) );
+
+  WebServer web((zypp::Pathname(TESTS_SRC_DIR)/"/zyppng/data/downloader").c_str(), 10001, withSSL );
+  BOOST_REQUIRE( web.start() );
+
+  zypp::filesystem::TmpFile targetFile;
+  zyppng::Url weburl (web.url());
+  weburl.setPathName("/handler/test.txt");
+  weburl.setQueryParam("auth", "basic");
+  weburl.setUsername("test");
+
+  // make sure the creds are already available
+  zypp::media::CredentialManager cm( repoManagerRoot.path() );
+  zypp::media::AuthData data ("test", "test");
+  data.setUrl( weburl );
+  cm.addCred( data );
+  cm.save();
+
+  zyppng::TransferSettings set = web.transferSettings();
+
+  web.addRequestHandler( "test.txt", createAuthHandler() );
+  web.addRequestHandler( "quit", [ &ev ]( WebServer::Request & ){ ev->quit();} );
+
+  {
+    // simply check by request count if the test was successful:
+    // if the proactive code adding the credentials to the first request is not executed we will
+    // have more than 1 request.
+    int reqCount = 0;
+    auto dispatcher = downloader->requestDispatcher();
+    dispatcher->sigDownloadStarted().connect([&]( zyppng::NetworkRequestDispatcher &, zyppng::NetworkRequest & ){
+      reqCount++;
+    });
+
+
+    auto dl = downloader->downloadFile( zyppng::DownloadSpec(weburl, targetFile.path())
+      .setMetalinkEnabled( false )
+      .setTransferSettings( set )
+    );
+
+    dl->sigFinished( ).connect([ &ev ]( zyppng::Download & ){
+      ev->quit();
+    });
+
+    dl->start();
+    ev->run();
+    BOOST_TEST_REQ_SUCCESS( dl );
+    BOOST_REQUIRE_EQUAL( reqCount, 1 );
+  }
+}
