@@ -41,11 +41,11 @@ namespace zypp
       const char *version = gpgme_check_version(NULL);
       if ( version )
       {
-	MIL << "Initialized libgpgme version: " << version << endl;
+        MIL << "Initialized libgpgme version: " << version << endl;
       }
       else
       {
-	MIL << "Initialized libgpgme with unknown version" << endl;
+        MIL << "Initialized libgpgme with unknown version" << endl;
       }
     }
 
@@ -86,7 +86,7 @@ namespace zypp
       str << "  "  << obj.not_imported		<< " keys not imported." << endl;
       for ( gpgme_import_status_t p = obj.imports; p; p = p->next )
       {
-	str << "  - "  << p->fpr << ": " << p->result << endl;
+        str << "  - "  << p->fpr << ": " << p->result << endl;
       }
       // In V.1.11: str << "  "  << obj.skipped_v3_keys	<< " skipped v3 keys." << endl;
       return str << "}";
@@ -115,6 +115,10 @@ namespace zypp
     std::list<std::string> readSignaturesFprs( const Pathname & signature_r )
     { return readSignaturesFprsOptVerify( signature_r ); }
 
+    /** Return all fingerprints found in \a signature_r. */
+    std::list<std::string> readSignaturesFprs( const ByteArray & signature_r )
+    { return readSignaturesFprsOptVerify( signature_r ); }
+
     /** Tries to verify the \a file_r using \a signature_r. */
     bool verifySignaturesFprs( const Pathname & file_r, const Pathname & signature_r )
     {
@@ -123,6 +127,9 @@ namespace zypp
       return verify;
     }
 
+    template< typename Callback >
+    bool importKey(GpgmeDataPtr &data, Callback &&calcDataSize );
+
     gpgme_ctx_t _ctx { nullptr };
     bool _volatile { false };	///< readKeyFromFile workaround bsc#1140670
 
@@ -133,6 +140,8 @@ namespace zypp
      * whether all signatures are good.
      */
     std::list<std::string> readSignaturesFprsOptVerify( const Pathname & signature_r, const Pathname & file_r = "/dev/null", bool * verify_r = nullptr );
+    std::list<std::string> readSignaturesFprsOptVerify( const ByteArray& keyData_r, const Pathname & file_r = "/dev/null", bool * verify_r = nullptr );
+    std::list<std::string> readSignaturesFprsOptVerify( GpgmeDataPtr &sigData, const Pathname & file_r = "/dev/null", bool * verify_r = nullptr );
   };
 
 std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify( const Pathname & signature_r, const Pathname & file_r, bool * verify_r )
@@ -141,21 +150,9 @@ std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify( const P
   if ( verify_r )
     *verify_r = false;
 
-
   if (!PathInfo( signature_r ).isExist())
     return std::list<std::string>();
 
-  FILEPtr dataFile(fopen(file_r.c_str(), "rb"), fclose);
-  if (!dataFile)
-    return std::list<std::string>();
-
-  GpgmeDataPtr fileData(nullptr, gpgme_data_release);
-  GpgmeErr err = gpgme_data_new_from_stream (&fileData.get(), dataFile.get());
-  if (err) {
-    ERR << err << endl;
-    return std::list<std::string>();
-  }
-
   FILEPtr sigFile(fopen(signature_r.c_str(), "rb"), fclose);
   if (!sigFile) {
     ERR << "Unable to open signature file '" << signature_r << "'" <<endl;
@@ -163,7 +160,43 @@ std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify( const P
   }
 
   GpgmeDataPtr sigData(nullptr, gpgme_data_release);
-  err = gpgme_data_new_from_stream (&sigData.get(), sigFile.get());
+  GpgmeErr err = gpgme_data_new_from_stream (&sigData.get(), sigFile.get());
+  if (err) {
+    ERR << err << endl;
+    return std::list<std::string>();
+  }
+
+  return readSignaturesFprsOptVerify( sigData, file_r, verify_r );
+}
+
+std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify( const ByteArray &keyData_r, const filesystem::Pathname &file_r, bool *verify_r )
+{
+  //lets be pessimistic
+  if ( verify_r )
+    *verify_r = false;
+
+  GpgmeDataPtr sigData(nullptr, gpgme_data_release);
+  GpgmeErr err = gpgme_data_new_from_mem(&sigData.get(), keyData_r.data(), keyData_r.size(), 1 );
+  if (err) {
+    ERR << err << endl;
+    return std::list<std::string>();
+  }
+
+  return readSignaturesFprsOptVerify( sigData, file_r, verify_r );
+}
+
+std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify(GpgmeDataPtr &sigData, const filesystem::Pathname &file_r, bool *verify_r)
+{
+  //lets be pessimistic
+  if ( verify_r )
+    *verify_r = false;
+
+  FILEPtr dataFile(fopen(file_r.c_str(), "rb"), fclose);
+  if (!dataFile)
+    return std::list<std::string>();
+
+  GpgmeDataPtr fileData(nullptr, gpgme_data_release);
+  GpgmeErr err = gpgme_data_new_from_stream (&fileData.get(), dataFile.get());
   if (err) {
     ERR << err << endl;
     return std::list<std::string>();
@@ -182,44 +215,54 @@ std::list<std::string> KeyManagerCtx::Impl::readSignaturesFprsOptVerify( const P
   }
 
   bool foundBadSignature = false;
+  bool foundGoodSignature = false;
   std::list<std::string> signatures;
   for ( gpgme_signature_t sig = res->signatures; sig; sig = sig->next ) {
 
     if ( sig->fpr )
     {
-      // bsc#1100427: With ibgpgme11-1.11.0 and if a recent gpg version was used
+      // bsc#1100427: With libgpgme11-1.11.0 and if a recent gpg version was used
       // to create the signature, the field may contain the full fingerprint, but
       // we're expected to return the ID.
       // [https://github.com/gpg/gpgme/commit/478d1650bbef84958ccce439fac982ef57b16cd0]
       std::string id( sig->fpr );
       if ( id.size() > 16 )
-	id = id.substr( id.size()-16 );
+        id = id.substr( id.size()-16 );
+
+      DBG << "Found signature with ID: " << id  << " in " << file_r << std::endl;
       signatures.push_back( std::move(id) );
     }
 
-    if ( sig->status != GPG_ERR_NO_ERROR )
-    {
-      if ( gpgme_err_code(sig->status) != GPG_ERR_KEY_EXPIRED )
+    if ( verify_r && sig->status != GPG_ERR_NO_ERROR ) {
+      const auto status = gpgme_err_code(sig->status);
+
+      // bsc#1180721: libgpgme started to return signatures of unknown keys, which breaks
+      // our workflow when verifying files that have multiple signatures, including some that are
+      // not in the trusted keyring. We should not fail if we have unknown or expired keys and at least a good one.
+      // We will however keep the behaviour of failing if we find a bad signatures even if others are good.
+      if ( status != GPG_ERR_KEY_EXPIRED && status != GPG_ERR_NO_PUBKEY )
       {
-	if ( !foundBadSignature )
-	  foundBadSignature = true;
-	if ( verify_r )
-	  WAR << "Failed signature check: " << file_r << " " << GpgmeErr(sig->status) << endl;
+        WAR << "Failed signature check: " << file_r << " " << GpgmeErr(sig->status) << endl;
+        if ( !foundBadSignature )
+          foundBadSignature = true;
       }
       else
       {
-	if ( verify_r )
-	  WAR << "Legacy: Ignore expired key: " << file_r << " " << GpgmeErr(sig->status) << endl;
+        WAR << "Legacy: Ignore expired or unknown key: " << file_r << " " << GpgmeErr(sig->status) << endl;
+        // for now treat expired keys as good signature
+        if ( status == GPG_ERR_KEY_EXPIRED )
+          foundGoodSignature = true;
       }
+    } else {
+      foundGoodSignature = true;
     }
   }
 
   if ( verify_r )
-    *verify_r = (!foundBadSignature);
+    *verify_r = (!foundBadSignature) && foundGoodSignature;
   return signatures;
 }
 
-
 KeyManagerCtx::KeyManagerCtx()
 : _pimpl( new Impl )
 {}
@@ -406,7 +449,11 @@ bool KeyManagerCtx::exportKey(const std::string &id, std::ostream &stream)
 
   //format as ascii armored
   gpgme_set_armor (_pimpl->_ctx, 1);
-  err = gpgme_op_export_keys (_pimpl->_ctx, keyarray, 0, out.get());
+  // bsc#1179222: Remove outdated self signatures when exporting the key.
+  // The keyring does not order the signatures when multiple versions of the
+  // same key are imported. Rpm however uses the 1st to compute the -release
+  // of the gpg-pubkey. So we export only the latest to get a proper-release.
+  err = gpgme_op_export_keys (_pimpl->_ctx, keyarray, GPGME_EXPORT_MODE_MINIMAL, out.get());
   if (!err) {
     int ret = gpgme_data_seek (out.get(), 0, SEEK_SET);
     if (ret) {
@@ -450,7 +497,28 @@ bool KeyManagerCtx::importKey(const Pathname &keyfile)
     return false;
   }
 
-  err = gpgme_op_import(_pimpl->_ctx, data.get());
+  return _pimpl->importKey( data, [&](){ return PathInfo(keyfile).size(); } );
+}
+
+bool KeyManagerCtx::importKey( const ByteArray &keydata )
+{
+  GpgmeDataPtr data(nullptr, gpgme_data_release);
+  GpgmeErr err;
+
+  err = gpgme_data_new_from_mem( &data.get(), keydata.data(), keydata.size(), 1);
+  if (err) {
+    ERR << "Error importing key: "<< err << endl;
+    return false;
+  }
+
+  return _pimpl->importKey( data, [&](){ return keydata.size(); } );
+}
+
+template<typename Callback>
+bool KeyManagerCtx::Impl::importKey(GpgmeDataPtr &data, Callback &&calcDataSize)
+{
+  GpgmeErr err;
+  err = gpgme_op_import( _ctx, data.get() );
   if (err) {
     ERR << "Error importing key: "<< err << endl;
     return false;
@@ -458,9 +526,9 @@ bool KeyManagerCtx::importKey(const Pathname &keyfile)
 
   // Work around bsc#1127220 [libgpgme] no error upon incomplete import due to signal received.
   // We need this error, otherwise RpmDb will report the missing keys as 'probably v3'.
-  if ( gpgme_import_result_t res = gpgme_op_import_result(_pimpl->_ctx) )
+  if ( gpgme_import_result_t res = gpgme_op_import_result(_ctx) )
   {
-    if ( ! res->considered && PathInfo(keyfile).size() )
+    if ( ! res->considered && std::forward<Callback>(calcDataSize)() )
     {
       DBG << *res << endl;
       ERR << "Error importing key: No keys considered (bsc#1127220, [libgpgme] signal received?)" << endl;
@@ -502,5 +570,8 @@ bool KeyManagerCtx::deleteKey(const std::string &id)
 std::list<std::string> KeyManagerCtx::readSignatureFingerprints(const Pathname &signature)
 { return _pimpl->readSignaturesFprs(signature); }
 
+std::list<std::string> KeyManagerCtx::readSignatureFingerprints(const ByteArray &keyData)
+{ return _pimpl->readSignaturesFprs(keyData); }
+
 } // namespace zypp
 ///////////////////////////////////////////////////////////////////
