@@ -24,7 +24,7 @@
 
 #include <zypp/ZConfig.h>
 #include <zypp/PathInfo.h>
-#include <zypp/parser/IniDict.h>
+#include <zypp-core/parser/IniDict>
 
 #undef  ZYPP_BASE_LOGGER_LOGGROUP
 #define ZYPP_BASE_LOGGER_LOGGROUP "zypp::VendorAttr"
@@ -58,6 +58,41 @@ namespace zypp
     bool equivalent( IdString lVendor, IdString rVendor ) const
     { return lVendor == rVendor || vendorMatchId( lVendor ) == vendorMatchId( rVendor ); }
 
+    /** Return whether two vendor strings should be treated as equivalent or are (suse/opensuse).*/
+    bool relaxedEquivalent( IdString lVendor, IdString rVendor ) const
+    {
+      if ( equivalent( lVendor, rVendor ) )
+        return true;
+
+      static const IdString suse { "suse" };
+      static const IdString opensuse { "opensuse" };
+      unsigned sid = vendorMatchId( suse );
+      unsigned oid = vendorMatchId( opensuse );
+      if ( sid == oid )
+        return false; // (suse/opensuse) are equivalent, so these are not
+
+      auto isSuse = [sid,oid]( unsigned v )-> bool { return v==sid || v==oid; };
+      return isSuse( vendorMatchId(lVendor) ) && isSuse( vendorMatchId(rVendor) );
+    }
+
+    unsigned foreachVendorList( std::function<bool(VendorList)> fnc_r ) const
+    {
+      std::map<unsigned,VendorList> lists;
+      for( const auto & el : _vendorGroupMap )
+        lists[el.second].push_back( el.first );
+
+      unsigned ret = 0;
+      for ( auto el : lists ) {
+        VendorList & vlist { el.second };
+        if ( vlist.empty() )
+          continue;
+        ++ret;
+        if ( fnc_r && !fnc_r( std::move(vlist) ) )
+          break;
+      }
+      return ret;
+    }
+
   private:
     using VendorGroupMap = std::map<std::string,unsigned>;
     VendorGroupMap _vendorGroupMap;	///< Vendor group definition. Equivalent groups share the same ID.
@@ -101,36 +136,36 @@ namespace zypp
       VendorMatchEntry & lcent( _vendorMatch[lcvendor] );
       if ( ! lcent )
       {
-	// Cache miss - check whether it belongs to a vendor group.
-	// Otherwise assign a new class ID to it.
-	unsigned myid = 0;
+        // Cache miss - check whether it belongs to a vendor group.
+        // Otherwise assign a new class ID to it.
+        unsigned myid = 0;
 
-	// bnc#812608: no prefix compare in opensuse namespace
-	if ( str::hasPrefix( lcvendor.c_str(), "opensuse" ) )
-	{
-	  if ( auto it = _vendorGroupMap.find( lcvendor.c_str() ); it != _vendorGroupMap.end() )
-	    myid = it->second;
-	}
-	else
-	{
-	  // Compare this entry with the global vendor map.
-	  // Reversed to get the longest prefix.
-	  for ( VendorGroupMap::const_reverse_iterator it = _vendorGroupMap.rbegin(); it != _vendorGroupMap.rend(); ++it )
-	  {
-	    if ( str::hasPrefix( lcvendor.c_str(), it->first ) ) {
-	      myid = it->second;
-	      break; // found
-	    }
-	  }
-	}
+        // bnc#812608: no prefix compare in opensuse namespace
+        if ( str::hasPrefix( lcvendor.c_str(), "opensuse" ) )
+        {
+          if ( auto it = _vendorGroupMap.find( lcvendor.c_str() ); it != _vendorGroupMap.end() )
+            myid = it->second;
+        }
+        else
+        {
+          // Compare this entry with the global vendor map.
+          // Reversed to get the longest prefix.
+          for ( VendorGroupMap::const_reverse_iterator it = _vendorGroupMap.rbegin(); it != _vendorGroupMap.rend(); ++it )
+          {
+            if ( str::hasPrefix( lcvendor.c_str(), it->first ) ) {
+              myid = it->second;
+              break; // found
+            }
+          }
+        }
 
-	if ( ! myid )
-	  myid = --_nextId; // get a new class ID
+        if ( ! myid )
+          myid = --_nextId; // get a new class ID
 
-	ent = lcent = myid; // remember the new DI
+        ent = lcent = myid; // remember the new DI
       }
       else
-	ent = lcent; // take the ID from the lowercased vendor string
+        ent = lcent; // take the ID from the lowercased vendor string
     }
     return ent;
   }
@@ -148,25 +183,25 @@ namespace zypp
 
       if ( _vendorGroupMap.count( vendor ) )
       {
-	unsigned joinId = _vendorGroupMap[vendor];
-	if ( targetId == _vendorGroupId + 1 ) {
-	  targetId = joinId;	// will extend the existing group
-	}
-	else if ( targetId != joinId ) {
-	  // yet another existing group -> join it into the target group
-	  for ( auto & el : _vendorGroupMap ) {
-	    if ( el.second == joinId )
-	      el.second = targetId;
-	  }
-	}
-	vendor.clear();	// no need to add it later
+        unsigned joinId = _vendorGroupMap[vendor];
+        if ( targetId == _vendorGroupId + 1 ) {
+          targetId = joinId;	// will extend the existing group
+        }
+        else if ( targetId != joinId ) {
+          // yet another existing group -> join it into the target group
+          for ( auto & el : _vendorGroupMap ) {
+            if ( el.second == joinId )
+              el.second = targetId;
+          }
+        }
+        vendor.clear();	// no need to add it later
       }
     }
 
     // Now add the new entries
     for ( std::string & vendor : vendorList_r ) {
       if ( ! vendor.empty() )
-	_vendorGroupMap[vendor] = targetId;
+        _vendorGroupMap[vendor] = targetId;
     }
 
     if ( targetId == _vendorGroupId + 1 )
@@ -228,11 +263,11 @@ namespace zypp
     }
 
     filesystem::dirForEach( dirname_r, filesystem::matchNoDots(),
-			    [this]( const Pathname & dir_r, const std::string & str_r )->bool
-			    {
-			      this->addVendorFile( dir_r/str_r );
-			      return true;
-			    }
+                            [this]( const Pathname & dir_r, const std::string & str_r )->bool
+                            {
+                              this->addVendorFile( dir_r/str_r );
+                              return true;
+                            }
     );
     return true;
   }
@@ -249,14 +284,14 @@ namespace zypp
     {
       if ( el.first == "vendors" )
       {
-	VendorList tmp;
-	strv::split( el.second, ",", strv::Trim::trim,
-		     [&tmp]( std::string_view word ) {
-		       if ( ! word.empty() )
-			 tmp.push_back( std::string(word) );
-		     } );
-	_addVendorList( std::move(tmp) );
-	break;
+        VendorList tmp;
+        strv::split( el.second, ",", strv::Trim::trim,
+                     [&tmp]( std::string_view word ) {
+                       if ( ! word.empty() )
+                         tmp.push_back( std::string(word) );
+                     } );
+        _addVendorList( std::move(tmp) );
+        break;
       }
     }
     return true;
@@ -265,6 +300,9 @@ namespace zypp
   void VendorAttr::_addVendorList( VendorList && vendorList_r )
   { _pimpl->addVendorList( std::move(vendorList_r) ); }
 
+  unsigned VendorAttr::foreachVendorList( std::function<bool(VendorList)> fnc_r ) const
+  { return _pimpl->foreachVendorList( std::move(fnc_r) ); }
+
 #if LEGACY(1722)
   bool VendorAttr::addVendorDirectory( const Pathname & dirname ) const
   { return const_cast<VendorAttr*>(this)->addVendorDirectory( dirname ); }
@@ -299,6 +337,19 @@ namespace zypp
   bool VendorAttr::equivalent( const PoolItem & lVendor, const PoolItem & rVendor ) const
   { return _pimpl->equivalent( lVendor.satSolvable().vendor(), rVendor.satSolvable().vendor() ); }
 
+
+  bool VendorAttr::relaxedEquivalent( IdString lVendor, IdString rVendor ) const
+  { return _pimpl->relaxedEquivalent( lVendor, rVendor );}
+
+  bool VendorAttr::relaxedEquivalent( const Vendor & lVendor, const Vendor & rVendor ) const
+  { return _pimpl->relaxedEquivalent( IdString( lVendor ), IdString( rVendor ) ); }
+
+  bool VendorAttr::relaxedEquivalent( sat::Solvable lVendor, sat::Solvable rVendor ) const
+  { return _pimpl->relaxedEquivalent( lVendor.vendor(), rVendor.vendor() ); }
+
+  bool VendorAttr::relaxedEquivalent( const PoolItem & lVendor, const PoolItem & rVendor ) const
+  { return _pimpl->relaxedEquivalent( lVendor.satSolvable().vendor(), rVendor.satSolvable().vendor() ); }
+
   //////////////////////////////////////////////////////////////////
 
   std::ostream & operator<<( std::ostream & str, const VendorAttr & obj )
