@@ -23,18 +23,20 @@ extern "C"
 #include <fstream>
 #include <zypp/base/LogTools.h>
 #include <zypp/base/IOStream.h>
-#include <zypp/base/InputStream.h>
+#include <zypp-core/base/InputStream>
 #include <zypp/base/String.h>
 #include <zypp/base/Regex.h>
 
 #include <zypp/ZConfig.h>
 #include <zypp/ZYppFactory.h>
 #include <zypp/PathInfo.h>
-#include <zypp/parser/IniDict.h>
+#include <zypp-core/parser/IniDict>
 
 #include <zypp/sat/Pool.h>
 #include <zypp/sat/detail/PoolImpl.h>
 
+#include <zypp-media/MediaConfig>
+
 using std::endl;
 using namespace zypp::filesystem;
 using namespace zypp::parser;
@@ -74,9 +76,9 @@ namespace zypp
 
       if ( architecture == Arch_i686 )
       {
-	// some CPUs report i686 but dont implement cx8 and cmov
-	// check for both flags in /proc/cpuinfo and downgrade
-	// to i586 if either is missing (cf bug #18885)
+        // some CPUs report i686 but dont implement cx8 and cmov
+        // check for both flags in /proc/cpuinfo and downgrade
+        // to i586 if either is missing (cf bug #18885)
         std::ifstream cpuinfo( "/proc/cpuinfo" );
         if ( cpuinfo )
         {
@@ -101,8 +103,8 @@ namespace zypp
       }
       else if ( architecture == Arch_sparc || architecture == Arch_sparc64 )
       {
-	// Check for sun4[vum] to get the real arch. (bug #566291)
-	std::ifstream cpuinfo( "/proc/cpuinfo" );
+        // Check for sun4[vum] to get the real arch. (bug #566291)
+        std::ifstream cpuinfo( "/proc/cpuinfo" );
         if ( cpuinfo )
         {
           for( iostr::EachLine in( cpuinfo ); in; in.next() )
@@ -133,35 +135,41 @@ namespace zypp
           ERR << "Cant open " << PathInfo("/proc/cpuinfo") << endl;
         }
       }
-      else if ( architecture == Arch_armv7l || architecture == Arch_armv6l )
+      else if ( architecture == Arch_armv8l || architecture == Arch_armv7l || architecture == Arch_armv6l )
       {
-	std::ifstream platform( "/etc/rpm/platform" );
-	if (platform)
-	{
-	  for( iostr::EachLine in( platform ); in; in.next() )
-	  {
-	    if ( str::hasPrefix( *in, "armv7hl-" ) )
-	    {
-	      architecture = Arch_armv7hl;
-	      WAR << "/etc/rpm/platform contains armv7hl-: architecture upgraded to '" << architecture << "'" << endl;
-	      break;
-	    }
-	    if ( str::hasPrefix( *in, "armv6hl-" ) )
-	    {
-	      architecture = Arch_armv6hl;
-	      WAR << "/etc/rpm/platform contains armv6hl-: architecture upgraded to '" << architecture << "'" << endl;
-	      break;
-	    }
-	  }
-	}
+        std::ifstream platform( "/etc/rpm/platform" );
+        if (platform)
+        {
+          for( iostr::EachLine in( platform ); in; in.next() )
+          {
+            if ( str::hasPrefix( *in, "armv8hl-" ) )
+            {
+              architecture = Arch_armv8hl;
+              WAR << "/etc/rpm/platform contains armv8hl-: architecture upgraded to '" << architecture << "'" << endl;
+              break;
+            }
+            if ( str::hasPrefix( *in, "armv7hl-" ) )
+            {
+              architecture = Arch_armv7hl;
+              WAR << "/etc/rpm/platform contains armv7hl-: architecture upgraded to '" << architecture << "'" << endl;
+              break;
+            }
+            if ( str::hasPrefix( *in, "armv6hl-" ) )
+            {
+              architecture = Arch_armv6hl;
+              WAR << "/etc/rpm/platform contains armv6hl-: architecture upgraded to '" << architecture << "'" << endl;
+              break;
+            }
+          }
+        }
       }
 #if __GLIBC_PREREQ (2,16)
       else if ( architecture == Arch_ppc64 )
       {
-	const char * platform = (const char *)getauxval( AT_PLATFORM );
-	int powerlvl;
-	if ( platform && sscanf( platform, "power%d", &powerlvl ) == 1 && powerlvl > 6 )
-	  architecture = Arch_ppc64p7;
+        const char * platform = (const char *)getauxval( AT_PLATFORM );
+        int powerlvl;
+        if ( platform && sscanf( platform, "power%d", &powerlvl ) == 1 && powerlvl > 6 )
+          architecture = Arch_ppc64p7;
       }
 #endif
       return architecture;
@@ -233,62 +241,65 @@ namespace zypp
   template<class Tp>
       struct Option
       {
-	typedef Tp value_type;
+        typedef Tp value_type;
 
-	/** No default ctor, explicit initialisation! */
-	Option( const value_type & initial_r )
-	  : _val( initial_r )
-	{}
+        /** No default ctor, explicit initialisation! */
+        Option( value_type initial_r )
+          : _val( std::move(initial_r) )
+        {}
 
-	/** Get the value.  */
-	const value_type & get() const
-	{ return _val; }
+        Option & operator=( value_type newval_r )
+        { set( std::move(newval_r) ); return *this; }
+
+        /** Get the value.  */
+        const value_type & get() const
+        { return _val; }
 
         /** Autoconversion to value_type.  */
         operator const value_type &() const
         { return _val; }
 
-	/** Set a new value.  */
-	void set( const value_type & newval_r )
-	{ _val = newval_r; }
+        /** Set a new value.  */
+        void set( value_type newval_r )
+        { _val = std::move(newval_r); }
 
-        /** Non-const reference to set a new value. */
-        value_type & ref()
-        { return _val; }
-
-	private:
-	  value_type _val;
+        private:
+          value_type _val;
       };
 
   /** Mutable option with initial value also remembering a config value. */
   template<class Tp>
       struct DefaultOption : public Option<Tp>
       {
-	typedef Tp         value_type;
-	typedef Option<Tp> option_type;
+        typedef Tp         value_type;
+        typedef Option<Tp> option_type;
 
-        DefaultOption( const value_type & initial_r )
-          : Option<Tp>( initial_r ), _default( initial_r )
+        explicit DefaultOption( value_type initial_r )
+        : Option<Tp>( initial_r )
+        , _default( std::move(initial_r) )
         {}
 
-	/** Reset value to the current default. */
-	void restoreToDefault()
-	{ this->set( _default.get() ); }
+        DefaultOption & operator=( value_type newval_r )
+        { this->set( std::move(newval_r) ); return *this; }
 
-	/** Reset value to a new default. */
-	void restoreToDefault( const value_type & newval_r )
-	{ setDefault( newval_r ); restoreToDefault(); }
+        /** Reset value to the current default. */
+        void restoreToDefault()
+        { this->set( _default.get() ); }
 
-	/** Get the current default value. */
-	const value_type & getDefault() const
-	{ return _default.get(); }
+        /** Reset value to a new default. */
+        void restoreToDefault( value_type newval_r )
+        { setDefault( std::move(newval_r) ); restoreToDefault(); }
 
-	/** Set a new default value. */
-	void setDefault( const value_type & newval_r )
-	{ _default.set( newval_r ); }
+        /** Get the current default value. */
+        const value_type & getDefault() const
+        { return _default.get(); }
 
-	private:
-	  option_type _default;
+        /** Set a new default value. */
+        void setDefault( value_type newval_r )
+        { _default.set( std::move(newval_r) ); }
+
+        private:
+          option_type _default;
       };
 
   ///////////////////////////////////////////////////////////////////
@@ -309,6 +320,10 @@ namespace zypp
         : _parsedZyppConf         	( override_r )
         , cfg_arch                	( defaultSystemArchitecture() )
         , cfg_textLocale          	( defaultTextLocale() )
+        , cfg_cache_path		{ "/var/cache/zypp" }
+        , cfg_metadata_path		{ "" }	// empty - follows cfg_cache_path
+        , cfg_solvfiles_path		{ "" }	// empty - follows cfg_cache_path
+        , cfg_packages_path		{ "" }	// empty - follows cfg_cache_path
         , updateMessagesNotify		( "" )
         , repo_add_probe          	( false )
         , repo_refresh_delay      	( 10 )
@@ -316,23 +331,18 @@ namespace zypp
         , download_use_deltarpm   	( true )
         , download_use_deltarpm_always  ( false )
         , download_media_prefer_download( true )
-	, download_mediaMountdir	( "/var/adm/mount" )
-        , download_max_concurrent_connections( 5 )
-        , download_min_download_speed	( 0 )
-        , download_max_download_speed	( 0 )
-        , download_max_silent_tries	( 5 )
-        , download_transfer_timeout	( 180 )
+        , download_mediaMountdir	( "/var/adm/mount" )
         , commit_downloadMode		( DownloadDefault )
-	, gpgCheck			( true )
-	, repoGpgCheck			( indeterminate )
-	, pkgGpgCheck			( indeterminate )
-	, solver_focus			( ResolverFocus::Default )
+        , gpgCheck			( true )
+        , repoGpgCheck			( indeterminate )
+        , pkgGpgCheck			( indeterminate )
+        , solver_focus			( ResolverFocus::Default )
         , solver_onlyRequires		( false )
         , solver_allowVendorChange	( false )
-	, solver_dupAllowDowngrade	( true )
-	, solver_dupAllowNameChange	( true )
-	, solver_dupAllowArchChange	( true )
-	, solver_dupAllowVendorChange	( true )
+        , solver_dupAllowDowngrade	( true )
+        , solver_dupAllowNameChange	( true )
+        , solver_dupAllowArchChange	( true )
+        , solver_dupAllowVendorChange	( true )
         , solver_cleandepsOnRemove	( false )
         , solver_upgradeTestcasesToKeep	( 2 )
         , solverUpgradeRemoveDroppedPackages( true )
@@ -355,6 +365,7 @@ namespace zypp
           INT << "Reconfigure to " << _parsedZyppConf << endl;
           ZConfig::instance()._pimpl.reset( this );
         }
+
         if ( PathInfo(_parsedZyppConf).isExist() )
         {
           parser::IniDict dict( _parsedZyppConf );
@@ -370,6 +381,10 @@ namespace zypp
             {
               std::string entry(it->first);
               std::string value(it->second);
+
+              if ( _mediaConf.setConfigValue( section, entry, value ) )
+                continue;
+
               //DBG << (*it).first << "=" << (*it).second << endl;
               if ( section == "main" )
               {
@@ -384,19 +399,19 @@ namespace zypp
                 }
                 else if ( entry == "cachedir" )
                 {
-                  cfg_cache_path = Pathname(value);
+                  cfg_cache_path.restoreToDefault( value );
                 }
                 else if ( entry == "metadatadir" )
                 {
-                  cfg_metadata_path = Pathname(value);
+                  cfg_metadata_path.restoreToDefault( value );
                 }
                 else if ( entry == "solvfilesdir" )
                 {
-                  cfg_solvfiles_path = Pathname(value);
+                  cfg_solvfiles_path.restoreToDefault( value );
                 }
                 else if ( entry == "packagesdir" )
                 {
-                  cfg_packages_path = Pathname(value);
+                  cfg_packages_path.restoreToDefault( value );
                 }
                 else if ( entry == "configdir" )
                 {
@@ -423,16 +438,16 @@ namespace zypp
                   str::strtonum(value, repo_refresh_delay);
                 }
                 else if ( entry == "repo.refresh.locales" )
-		{
-		  std::vector<std::string> tmp;
-		  str::split( value, back_inserter( tmp ), ", \t" );
+                {
+                  std::vector<std::string> tmp;
+                  str::split( value, back_inserter( tmp ), ", \t" );
 
-		  boost::function<Locale(const std::string &)> transform(
-		    [](const std::string & str_r)->Locale{ return Locale(str_r); }
-		  );
-		  repoRefreshLocales.insert( make_transform_iterator( tmp.begin(), transform ),
-					     make_transform_iterator( tmp.end(), transform ) );
-		}
+                  boost::function<Locale(const std::string &)> transform(
+                    [](const std::string & str_r)->Locale{ return Locale(str_r); }
+                  );
+                  repoRefreshLocales.insert( make_transform_iterator( tmp.begin(), transform ),
+                                             make_transform_iterator( tmp.end(), transform ) );
+                }
                 else if ( entry == "download.use_deltarpm" )
                 {
                   download_use_deltarpm = str::strToBool( value, download_use_deltarpm );
@@ -441,54 +456,30 @@ namespace zypp
                 {
                   download_use_deltarpm_always = str::strToBool( value, download_use_deltarpm_always );
                 }
-		else if ( entry == "download.media_preference" )
+                else if ( entry == "download.media_preference" )
                 {
-		  download_media_prefer_download.restoreToDefault( str::compareCI( value, "volatile" ) != 0 );
+                  download_media_prefer_download.restoreToDefault( str::compareCI( value, "volatile" ) != 0 );
                 }
-
-		else if ( entry == "download.media_mountdir" )
+                else if ( entry == "download.media_mountdir" )
                 {
-		  download_mediaMountdir.restoreToDefault( Pathname(value) );
-                }
-
-                else if ( entry == "download.max_concurrent_connections" )
-                {
-                  str::strtonum(value, download_max_concurrent_connections);
-                }
-                else if ( entry == "download.min_download_speed" )
-                {
-                  str::strtonum(value, download_min_download_speed);
-                }
-                else if ( entry == "download.max_download_speed" )
-                {
-                  str::strtonum(value, download_max_download_speed);
-                }
-                else if ( entry == "download.max_silent_tries" )
-                {
-                  str::strtonum(value, download_max_silent_tries);
-                }
-                else if ( entry == "download.transfer_timeout" )
-                {
-                  str::strtonum(value, download_transfer_timeout);
-		  if ( download_transfer_timeout < 0 )		download_transfer_timeout = 0;
-		  else if ( download_transfer_timeout > 3600 )	download_transfer_timeout = 3600;
+                  download_mediaMountdir.restoreToDefault( Pathname(value) );
                 }
                 else if ( entry == "commit.downloadMode" )
                 {
                   commit_downloadMode.set( deserializeDownloadMode( value ) );
                 }
                 else if ( entry == "gpgcheck" )
-		{
-		  gpgCheck.restoreToDefault( str::strToBool( value, gpgCheck ) );
-		}
-		else if ( entry == "repo_gpgcheck" )
-		{
-		  repoGpgCheck.restoreToDefault( str::strToTriBool( value ) );
-		}
-		else if ( entry == "pkg_gpgcheck" )
-		{
-		  pkgGpgCheck.restoreToDefault( str::strToTriBool( value ) );
-		}
+                {
+                  gpgCheck.restoreToDefault( str::strToBool( value, gpgCheck ) );
+                }
+                else if ( entry == "repo_gpgcheck" )
+                {
+                  repoGpgCheck.restoreToDefault( str::strToTriBool( value ) );
+                }
+                else if ( entry == "pkg_gpgcheck" )
+                {
+                  pkgGpgCheck.restoreToDefault( str::strToTriBool( value ) );
+                }
                 else if ( entry == "vendordir" )
                 {
                   cfg_vendor_path = Pathname(value);
@@ -503,7 +494,7 @@ namespace zypp
                 }
                 else if ( entry == "solver.focus" )
                 {
-		  fromString( value, solver_focus );
+                  fromString( value, solver_focus );
                 }
                 else if ( entry == "solver.onlyRequires" )
                 {
@@ -514,21 +505,21 @@ namespace zypp
                   solver_allowVendorChange.set( str::strToBool( value, solver_allowVendorChange ) );
                 }
                 else if ( entry == "solver.dupAllowDowngrade" )
-		{
-		  solver_dupAllowDowngrade.set( str::strToBool( value, solver_dupAllowDowngrade ) );
-		}
-		else if ( entry == "solver.dupAllowNameChange" )
-		{
-		  solver_dupAllowNameChange.set( str::strToBool( value, solver_dupAllowNameChange ) );
-		}
-		else if ( entry == "solver.dupAllowArchChange" )
-		{
-		  solver_dupAllowArchChange.set( str::strToBool( value, solver_dupAllowArchChange ) );
-		}
-		else if ( entry == "solver.dupAllowVendorChange" )
-		{
-		  solver_dupAllowVendorChange.set( str::strToBool( value, solver_dupAllowVendorChange ) );
-		}
+                {
+                  solver_dupAllowDowngrade.set( str::strToBool( value, solver_dupAllowDowngrade ) );
+                }
+                else if ( entry == "solver.dupAllowNameChange" )
+                {
+                  solver_dupAllowNameChange.set( str::strToBool( value, solver_dupAllowNameChange ) );
+                }
+                else if ( entry == "solver.dupAllowArchChange" )
+                {
+                  solver_dupAllowArchChange.set( str::strToBool( value, solver_dupAllowArchChange ) );
+                }
+                else if ( entry == "solver.dupAllowVendorChange" )
+                {
+                  solver_dupAllowVendorChange.set( str::strToBool( value, solver_dupAllowVendorChange ) );
+                }
                 else if ( entry == "solver.cleandepsOnRemove" )
                 {
                   solver_cleandepsOnRemove.set( str::strToBool( value, solver_cleandepsOnRemove ) );
@@ -551,7 +542,7 @@ namespace zypp
                 }
                 else if ( entry == "multiversion" )
                 {
-		  MultiversionSpec & defSpec( _multiversionMap.getDefaultSpec() );
+                  MultiversionSpec & defSpec( _multiversionMap.getDefaultSpec() );
                   str::splitEscaped( value, std::inserter( defSpec, defSpec.end() ), ", \t" );
                 }
                 else if ( entry == "locksfile.path" )
@@ -587,13 +578,15 @@ namespace zypp
                 {
                   history_log_path = Pathname(value);
                 }
-                else if ( entry == "credentials.global.dir" )
+                else if ( entry == "techpreview.ZYPP_SINGLE_RPMTRANS" )
                 {
-                  credentials_global_dir_path = Pathname(value);
+                  DBG << "techpreview.ZYPP_SINGLE_RPMTRANS=" << value << endl;
+                  ::setenv( "ZYPP_SINGLE_RPMTRANS", value.c_str(), 1 );
                 }
-                else if ( entry == "credentials.global.file" )
+                else if ( entry == "techpreview.ZYPP_MEDIANETWORK" )
                 {
-                  credentials_global_file_path = Pathname(value);
+                  DBG << "techpreview.ZYPP_MEDIANETWORK=" << value << endl;
+                  ::setenv( "ZYPP_MEDIANETWORK", value.c_str(), 1 );
                 }
               }
             }
@@ -630,10 +623,10 @@ namespace zypp
     Arch     cfg_arch;
     Locale   cfg_textLocale;
 
-    Pathname cfg_cache_path;
-    Pathname cfg_metadata_path;
-    Pathname cfg_solvfiles_path;
-    Pathname cfg_packages_path;
+    DefaultOption<Pathname> cfg_cache_path;	// Settings from the config file are also remembered
+    DefaultOption<Pathname> cfg_metadata_path;	// 'default'. Cleanup in RepoManager e.g needs to tell
+    DefaultOption<Pathname> cfg_solvfiles_path;	// whether settings in effect are config values or
+    DefaultOption<Pathname> cfg_packages_path;	// custom settings applied vie set...Path().
 
     Pathname cfg_config_path;
     Pathname cfg_known_repos_path;
@@ -661,12 +654,6 @@ namespace zypp
     DefaultOption<bool> download_media_prefer_download;
     DefaultOption<Pathname> download_mediaMountdir;
 
-    int download_max_concurrent_connections;
-    int download_min_download_speed;
-    int download_max_download_speed;
-    int download_max_silent_tries;
-    int download_transfer_timeout;
-
     Option<DownloadMode> commit_downloadMode;
 
     DefaultOption<bool>		gpgCheck;
@@ -695,13 +682,14 @@ namespace zypp
     target::rpm::RpmInstFlags rpmInstallFlags;
 
     Pathname history_log_path;
-    Pathname credentials_global_dir_path;
-    Pathname credentials_global_file_path;
 
     std::string userData;
 
     Option<Pathname> pluginsPath;
 
+    /* Other config singleton instances */
+    MediaConfig &_mediaConf = MediaConfig::instance();
+
   private:
     // HACK for bnc#906096: let pool re-evaluate multiversion spec
     // if target root changes. ZConfig returns data sensitive to
@@ -714,70 +702,73 @@ namespace zypp
 
       MultiversionSpec & getSpec( Pathname root_r, const Impl & zConfImpl_r )	// from system at root
       {
-	// _specMap[]     - the plain zypp.conf value
-	// _specMap[/]    - combine [] and multiversion.d scan
-	// _specMap[root] - scan root/zypp.conf and root/multiversion.d
+        // _specMap[]     - the plain zypp.conf value
+        // _specMap[/]    - combine [] and multiversion.d scan
+        // _specMap[root] - scan root/zypp.conf and root/multiversion.d
 
-	if ( root_r.empty() )
-	  root_r = "/";
-	bool cacheHit = _specMap.count( root_r );
-	MultiversionSpec & ret( _specMap[root_r] );	// creates new entry on the fly
+        if ( root_r.empty() )
+          root_r = "/";
+        bool cacheHit = _specMap.count( root_r );
+        MultiversionSpec & ret( _specMap[root_r] );	// creates new entry on the fly
 
-	if ( ! cacheHit )
-	{
-	  if ( root_r == "/" )
-	    ret.swap( _specMap[Pathname()] );		// original zypp.conf
-	  else
-	    scanConfAt( root_r, ret, zConfImpl_r );	// scan zypp.conf at root_r
-	  scanDirAt( root_r, ret, zConfImpl_r );	// add multiversion.d at root_r
-	  using zypp::operator<<;
-	  MIL << "MultiversionSpec '" << root_r << "' = " << ret << endl;
-	}
-	return ret;
+        if ( ! cacheHit )
+        {
+          // bsc#1193488: If no (/root)/.../zypp.conf exists use the default zypp.conf
+          // multiversion settings. It is a legacy that the packaged multiversion setting
+          // in zypp.conf (the kernel) may differ from the builtin default (empty).
+          // But we want a missing config to behave similar to the default one, otherwise
+          // a bare metal install easily runs into trouble.
+          if ( root_r == "/" || scanConfAt( root_r, ret, zConfImpl_r ) == 0 )
+            ret = _specMap[Pathname()];
+          scanDirAt( root_r, ret, zConfImpl_r );	// add multiversion.d at root_r
+          using zypp::operator<<;
+          MIL << "MultiversionSpec '" << root_r << "' = " << ret << endl;
+        }
+        return ret;
       }
 
       MultiversionSpec & getDefaultSpec()	// Spec from zypp.conf parsing; called before any getSpec
       {	return _specMap[Pathname()]; }
 
     private:
-      void scanConfAt( const Pathname root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
+      int scanConfAt( const Pathname root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
       {
-	static const str::regex rx( "^multiversion *= *(.*)" );
-	str::smatch what;
-	iostr::simpleParseFile( InputStream( Pathname::assertprefix( root_r, _autodetectZyppConfPath() ) ),
-				[&]( int num_r, std::string line_r )->bool
-				{
-				  if ( line_r[0] == 'm' && str::regex_match( line_r, what, rx ) )
-				  {
-				    str::splitEscaped( what[1], std::inserter( spec_r, spec_r.end() ), ", \t" );
-				    return false;	// stop after match
-				  }
-				  return true;
-				} );
+        static const str::regex rx( "^multiversion *= *(.*)" );
+        str::smatch what;
+        return iostr::simpleParseFile( InputStream( Pathname::assertprefix( root_r, _autodetectZyppConfPath() ) ),
+                                [&]( int num_r, std::string line_r )->bool
+                                {
+                                  if ( line_r[0] == 'm' && str::regex_match( line_r, what, rx ) )
+                                  {
+                                    str::splitEscaped( what[1], std::inserter( spec_r, spec_r.end() ), ", \t" );
+                                    return false;	// stop after match
+                                  }
+                                  return true;
+                                } );
       }
 
       void scanDirAt( const Pathname root_r, MultiversionSpec & spec_r, const Impl & zConfImpl_r )
       {
-	// NOTE:  Actually we'd need to scan and use the root_r! zypp.conf values.
-	Pathname multiversionDir( zConfImpl_r.cfg_multiversion_path );
-	if ( multiversionDir.empty() )
-	  multiversionDir = ( zConfImpl_r.cfg_config_path.empty()
-			    ? Pathname("/etc/zypp")
-			    : zConfImpl_r.cfg_config_path ) / "multiversion.d";
+        // NOTE:  Actually we'd need to scan and use the root_r! zypp.conf values.
+        Pathname multiversionDir( zConfImpl_r.cfg_multiversion_path );
+        if ( multiversionDir.empty() )
+          multiversionDir = ( zConfImpl_r.cfg_config_path.empty()
+                            ? Pathname("/etc/zypp")
+                            : zConfImpl_r.cfg_config_path ) / "multiversion.d";
 
-	filesystem::dirForEach( Pathname::assertprefix( root_r, multiversionDir ),
-				[&spec_r]( const Pathname & dir_r, const char *const & name_r )->bool
-				{
-				  MIL << "Parsing " << dir_r/name_r << endl;
-				  iostr::simpleParseFile( InputStream( dir_r/name_r ),
-							  [&spec_r]( int num_r, std::string line_r )->bool
-							  {
-							    DBG << "  found " << line_r << endl;
-							    spec_r.insert( std::move(line_r) );
-							    return true;
-							  } );
-				  return true;
-				} );
+        filesystem::dirForEach( Pathname::assertprefix( root_r, multiversionDir ),
+                                [&spec_r]( const Pathname & dir_r, const char *const & name_r )->bool
+                                {
+                                  MIL << "Parsing " << dir_r/name_r << endl;
+                                  iostr::simpleParseFile( InputStream( dir_r/name_r ),
+                                                          [&spec_r]( int num_r, std::string line_r )->bool
+                                                          {
+                                                            DBG << "  found " << line_r << endl;
+                                                            spec_r.insert( std::move(line_r) );
+                                                            return true;
+                                                          } );
+                                  return true;
+                                } );
       }
 
     private:
@@ -879,7 +870,7 @@ namespace zypp
     {
       WAR << "Overriding text locale (" << _pimpl->cfg_textLocale << "): " << locale_r << endl;
       _pimpl->cfg_textLocale = locale_r;
-#warning prefer signal
+      // Propagate changes
       sat::Pool::instance().setTextLocale( locale_r );
     }
   }
@@ -900,8 +891,8 @@ namespace zypp
     {
       if ( *ch < ' ' && *ch != '\t' )
       {
-	ERR << "New user data string rejectded: char " << (int)*ch << " at position " <<  (ch - str_r.begin()) << endl;
-	return false;
+        ERR << "New user data string rejectded: char " << (int)*ch << " at position " <<  (ch - str_r.begin()) << endl;
+        return false;
       }
     }
     MIL << "Set user data string to '" << str_r << "'" << endl;
@@ -913,14 +904,13 @@ namespace zypp
 
   Pathname ZConfig::repoCachePath() const
   {
-    return ( _pimpl->cfg_cache_path.empty()
-             ? Pathname("/var/cache/zypp") : _pimpl->cfg_cache_path );
+    return ( _pimpl->cfg_cache_path.get().empty()
+             ? Pathname("/var/cache/zypp") : _pimpl->cfg_cache_path.get() );
   }
 
   Pathname ZConfig::pubkeyCachePath() const
   {
-    return ( _pimpl->cfg_cache_path.empty()
-             ? Pathname("/var/cache/zypp/pubkeys") : _pimpl->cfg_cache_path/"pubkeys" );
+    return repoCachePath()/"pubkeys";
   }
 
   void ZConfig::setRepoCachePath(const zypp::filesystem::Pathname &path_r)
@@ -930,8 +920,8 @@ namespace zypp
 
   Pathname ZConfig::repoMetadataPath() const
   {
-    return ( _pimpl->cfg_metadata_path.empty()
-        ? (repoCachePath()/"raw") : _pimpl->cfg_metadata_path );
+    return ( _pimpl->cfg_metadata_path.get().empty()
+        ? (repoCachePath()/"raw") : _pimpl->cfg_metadata_path.get() );
   }
 
   void ZConfig::setRepoMetadataPath(const zypp::filesystem::Pathname &path_r)
@@ -941,8 +931,8 @@ namespace zypp
 
   Pathname ZConfig::repoSolvfilesPath() const
   {
-    return ( _pimpl->cfg_solvfiles_path.empty()
-        ? (repoCachePath()/"solv") : _pimpl->cfg_solvfiles_path );
+    return ( _pimpl->cfg_solvfiles_path.get().empty()
+        ? (repoCachePath()/"solv") : _pimpl->cfg_solvfiles_path.get() );
   }
 
   void ZConfig::setRepoSolvfilesPath(const zypp::filesystem::Pathname &path_r)
@@ -952,8 +942,8 @@ namespace zypp
 
   Pathname ZConfig::repoPackagesPath() const
   {
-    return ( _pimpl->cfg_packages_path.empty()
-        ? (repoCachePath()/"packages") : _pimpl->cfg_packages_path );
+    return ( _pimpl->cfg_packages_path.get().empty()
+        ? (repoCachePath()/"packages") : _pimpl->cfg_packages_path.get() );
   }
 
   void ZConfig::setRepoPackagesPath(const zypp::filesystem::Pathname &path_r)
@@ -961,6 +951,18 @@ namespace zypp
     _pimpl->cfg_packages_path = path_r;
   }
 
+  Pathname ZConfig::builtinRepoCachePath() const
+  { return _pimpl->cfg_cache_path.getDefault().empty() ? Pathname("/var/cache/zypp") : _pimpl->cfg_cache_path.getDefault(); }
+
+  Pathname ZConfig::builtinRepoMetadataPath() const
+  { return _pimpl->cfg_metadata_path.getDefault().empty() ? (builtinRepoCachePath()/"raw") : _pimpl->cfg_metadata_path.getDefault(); }
+
+  Pathname ZConfig::builtinRepoSolvfilesPath() const
+  { return _pimpl->cfg_solvfiles_path.getDefault().empty() ? (builtinRepoCachePath()/"solv") : _pimpl->cfg_solvfiles_path.getDefault(); }
+
+  Pathname ZConfig::builtinRepoPackagesPath() const
+  { return _pimpl->cfg_packages_path.getDefault().empty() ? (builtinRepoCachePath()/"packages") : _pimpl->cfg_packages_path.getDefault(); }
+
   ///////////////////////////////////////////////////////////////////
 
   Pathname ZConfig::configPath() const
@@ -1038,19 +1040,19 @@ namespace zypp
   { _pimpl->download_media_prefer_download.restoreToDefault(); }
 
   long ZConfig::download_max_concurrent_connections() const
-  { return _pimpl->download_max_concurrent_connections; }
+  { return _pimpl->_mediaConf.download_max_concurrent_connections(); }
 
   long ZConfig::download_min_download_speed() const
-  { return _pimpl->download_min_download_speed; }
+  { return _pimpl->_mediaConf.download_min_download_speed(); }
 
   long ZConfig::download_max_download_speed() const
-  { return _pimpl->download_max_download_speed; }
+  { return _pimpl->_mediaConf.download_max_download_speed(); }
 
   long ZConfig::download_max_silent_tries() const
-  { return _pimpl->download_max_silent_tries; }
+  { return _pimpl->_mediaConf.download_max_silent_tries(); }
 
   long ZConfig::download_transfer_timeout() const
-  { return _pimpl->download_transfer_timeout; }
+  { return _pimpl->_mediaConf.download_transfer_timeout(); }
 
   Pathname ZConfig::download_mediaMountdir() const		{ return _pimpl->download_mediaMountdir; }
   void ZConfig::set_download_mediaMountdir( Pathname newval_r )	{ _pimpl->download_mediaMountdir.set( std::move(newval_r) ); }
@@ -1161,14 +1163,12 @@ namespace zypp
 
   Pathname ZConfig::credentialsGlobalDir() const
   {
-    return ( _pimpl->credentials_global_dir_path.empty() ?
-        Pathname("/etc/zypp/credentials.d") : _pimpl->credentials_global_dir_path );
+    return _pimpl->_mediaConf.credentialsGlobalDir();
   }
 
   Pathname ZConfig::credentialsGlobalFile() const
   {
-    return ( _pimpl->credentials_global_file_path.empty() ?
-        Pathname("/etc/zypp/credentials.cat") : _pimpl->credentials_global_file_path );
+    return _pimpl->_mediaConf.credentialsGlobalFile();
   }
 
   ///////////////////////////////////////////////////////////////////
