@@ -16,33 +16,37 @@
 #include <set>
 #include <string>
 
-#include "zypp/APIConfig.h"
-#include "zypp/base/NonCopyable.h"
-#include "zypp/base/PtrTypes.h"
+#include <zypp/APIConfig.h>
+#include <zypp/base/NonCopyable.h>
+#include <zypp/base/PtrTypes.h>
 
-#include "zypp/Arch.h"
-#include "zypp/Locale.h"
-#include "zypp/Pathname.h"
-#include "zypp/IdString.h"
-#include "zypp/TriBool.h"
-
-#include "zypp/DownloadMode.h"
-#include "zypp/target/rpm/RpmFlags.h"
+#include <zypp/Arch.h>
+#include <zypp/Locale.h>
+#include <zypp/Pathname.h>
+#include <zypp/IdString.h>
+#include <zypp/TriBool.h>
+#include <zypp/ResolverFocus.h>
+#include <zypp/DownloadMode.h>
+#include <zypp/target/rpm/RpmFlags.h>
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
 { /////////////////////////////////////////////////////////////////
 
+  class RepoManager;
+
   ///////////////////////////////////////////////////////////////////
   //
   //	CLASS NAME : ZConfig
   //
   /** Interim helper class to collect global options and settings.
    * Use it to avoid hardcoded values and calls to getZYpp() just
-   * to retrieve some value like architecture, languages or tmppath.
+   * to retrieve some value like architecture, languages or temppath.
    *
    * It reads /etc/zypp/zypp.conf, the filename can be overridden by
    * setting the ZYPP_CONF environment variable to a different file.
+   * Config values for \ref MediaConfig are forwarded to the MediaConfig
+   * singleton.
    *
    * Note, if you add settings to this file, please follow the following
    * convention:
@@ -74,7 +78,7 @@ namespace zypp
       Pathname systemRoot() const;
 
       /** The RepoManager root directory.
-       *  Returns the same as \sa systemRoot() if not explicitely set.
+       *  Returns the same as \ref systemRoot() if not explicitly set.
        */
       Pathname repoManagerRoot() const;
 
@@ -94,7 +98,7 @@ namespace zypp
       /** Override the zypp system architecture.
        * This is useful for test scenarious. <b>But be warned</b>, zypp does
        * not expect the system architecture to change at runtime. So
-       * set it at the verry beginning before you acess any other
+       * set it at the very beginning before you access any other
        * zypp component.
       */
       void setSystemArchitecture( const Arch & arch_r );
@@ -104,7 +108,7 @@ namespace zypp
       { setSystemArchitecture( defaultSystemArchitecture() ); }
 
     public:
-      /** The autodetected prefered locale for translated texts.
+      /** The autodetected preferred locale for translated texts.
        */
       static Locale defaultTextLocale();
 
@@ -112,7 +116,7 @@ namespace zypp
        */
       Locale textLocale() const;
 
-      /** Set the prefered locale for translated texts. */
+      /** Set the preferred locale for translated texts. */
       void setTextLocale( const Locale & locale_r );
 
       /** Reset the locale for translated texts to the default. */
@@ -183,6 +187,7 @@ namespace zypp
       */
       Pathname repoPackagesPath() const;
 
+
       /**
        * Set a new \a path as the default repo cache path
        */
@@ -360,6 +365,11 @@ namespace zypp
        */
       Pathname vendorPath() const;
 
+      /**
+       * The resolvers general attitude when resolving jobs.
+       */
+      ResolverFocus solver_focus() const;
+
       /**
        * Solver regards required packages,patterns,... only
        */
@@ -529,11 +539,28 @@ namespace zypp
        */
       Pathname pluginsPath() const;
 
+      /*!
+       * Defaults to a empty string, if no keep spec is defined no kernels are removed
+       */
+      std::string multiversionKernels() const;
+
       //@}
+
     public:
       class Impl;
       /** Dtor */
       ~ZConfig();
+  private:
+      friend class RepoManager;
+      /** The builtin config file value. */
+      Pathname builtinRepoCachePath() const;
+      /** The builtin config file value. */
+      Pathname builtinRepoMetadataPath() const;
+      /** The builtin config file value. */
+      Pathname builtinRepoSolvfilesPath() const;
+      /** The builtin config file value. */
+      Pathname builtinRepoPackagesPath() const;
+
   private:
       friend class Impl;
       /** Default ctor. */
