@@ -21,8 +21,8 @@
 #include <zypp/Package.h>
 #include <zypp/Patch.h>
 #include <zypp/Url.h>
-#include <zypp/ProgressData.h>
-#include <zypp/media/MediaUserAuth.h>
+#include <zypp-core/ui/ProgressData>
+#include <zypp-media/auth/AuthData>
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -161,8 +161,8 @@ namespace zypp
 
       virtual Action problem(
         Resolvable::constPtr /*resolvable_ptr*/
-	, Error /*error*/
-	, const std::string &/*description*/
+        , Error /*error*/
+        , const std::string &/*description*/
       ) { return ABORT; }
 
 
@@ -176,7 +176,7 @@ namespace zypp
        *
        *  Userdata accepted:
        * \param "Action"	DownloadResolvableReport::Action user advice how to behave on error (ABORT).
-       *                        If you set just an empty value here, a default probelm report will be triggered.
+       *                        If you set just an empty value here, a default problem report will be triggered.
        *
        * Legacy data:
        * \param "Package"	Replaced by \c "ResObject" in 16.10.0. Package::constPtr of the package (\c nullptr in case of a SrcPackage)
@@ -384,14 +384,14 @@ namespace zypp
 
         virtual Action problem(
           const Url &/*file*/
-  	  , Error /*error*/
-  	  , const std::string &/*description*/
+          , Error /*error*/
+          , const std::string &/*description*/
         ) { return ABORT; }
 
         virtual void finish(
           const Url &/*file*/
           , Error /*error*/
-	  , const std::string &/*reason*/
+          , const std::string &/*reason*/
         ) {}
     };
 
@@ -407,7 +407,7 @@ namespace zypp
        *        data. As an input parameter auth_data can be prefilled with
        *        username (from previous try) or auth_type (available
        *        authentication methods aquired from server (only CurlAuthData)).
-       *        As an output parameter it serves for sending username, pasword
+       *        As an output parameter it serves for sending username, password
        *        or other special data (derived AuthData objects).
        * \return true if user chooses to continue with authentication,
        *         false otherwise
@@ -518,14 +518,14 @@ namespace zypp
         enum Action {
           ABORT,  // abort and return error
           RETRY,	// retry
-	  IGNORE	// ignore the failure
+          IGNORE	// ignore the failure
         };
 
         enum Error {
-	  NO_ERROR,
+          NO_ERROR,
           NOT_FOUND, 	// the requested Url was not found
-	  IO,		// IO error
-	  INVALID		// th resolvable is invalid
+          IO,		// IO error
+          INVALID		// th resolvable is invalid
         };
 
         // the level of RPM pushing
@@ -537,7 +537,7 @@ namespace zypp
         };
 
         virtual void start(
-	  Resolvable::constPtr /*resolvable*/
+          Resolvable::constPtr /*resolvable*/
         ) {}
 
         virtual bool progress(int /*value*/, Resolvable::constPtr /*resolvable*/)
@@ -545,17 +545,25 @@ namespace zypp
 
         virtual Action problem(
           Resolvable::constPtr /*resolvable*/
-  	  , Error /*error*/
-   	  , const std::string &/*description*/
-	  , RpmLevel /*level*/
+          , Error /*error*/
+          , const std::string &/*description*/
+          , RpmLevel /*level*/
         ) { return ABORT; }
 
         virtual void finish(
           Resolvable::constPtr /*resolvable*/
           , Error /*error*/
-	  , const std::string &/*reason*/
-	  , RpmLevel /*level*/
+          , const std::string &/*reason*/
+          , RpmLevel /*level*/
         ) {}
+
+        /** "rpmout/installpkg": Additional rpm output (sent immediately).
+         * Data:
+         * solvable : satSolvable processed
+         * line     : std::reference_wrapper<const std::string>
+         * lineno   : unsigned
+         */
+        static const UserData::ContentType contentRpmout;
       };
 
       // progress for removing a resolvable
@@ -564,18 +572,18 @@ namespace zypp
         enum Action {
           ABORT,  // abort and return error
           RETRY,	// retry
-	  IGNORE	// ignore the failure
+          IGNORE	// ignore the failure
         };
 
         enum Error {
-	  NO_ERROR,
+          NO_ERROR,
           NOT_FOUND, 	// the requested Url was not found
-	  IO,		// IO error
-	  INVALID		// th resolvable is invalid
+          IO,		// IO error
+          INVALID		// th resolvable is invalid
         };
 
         virtual void start(
-	  Resolvable::constPtr /*resolvable*/
+          Resolvable::constPtr /*resolvable*/
         ) {}
 
         virtual bool progress(int /*value*/, Resolvable::constPtr /*resolvable*/)
@@ -583,15 +591,20 @@ namespace zypp
 
         virtual Action problem(
           Resolvable::constPtr /*resolvable*/
-  	  , Error /*error*/
-  	  , const std::string &/*description*/
+          , Error /*error*/
+          , const std::string &/*description*/
         ) { return ABORT; }
 
         virtual void finish(
           Resolvable::constPtr /*resolvable*/
           , Error /*error*/
-	  , const std::string &/*reason*/
+          , const std::string &/*reason*/
         ) {}
+
+        /** "rpmout/removepkg": Additional rpm output (sent immediately).
+         * For data \see \ref InstallResolvableReport::contentRpmout
+         */
+        static const UserData::ContentType contentRpmout;
       };
 
       // progress for rebuilding the database
@@ -600,12 +613,12 @@ namespace zypp
         enum Action {
           ABORT,  // abort and return error
           RETRY,	// retry
-	  IGNORE	// ignore the failure
+          IGNORE	// ignore the failure
         };
 
         enum Error {
-	  NO_ERROR,
-	  FAILED		// failed to rebuild
+          NO_ERROR,
+          FAILED		// failed to rebuild
         };
 
         virtual void start(Pathname /*path*/) {}
@@ -614,15 +627,15 @@ namespace zypp
         { return true; }
 
         virtual Action problem(
-	  Pathname /*path*/
-  	 , Error /*error*/
-  	 , const std::string &/*description*/
+          Pathname /*path*/
+         , Error /*error*/
+         , const std::string &/*description*/
         ) { return ABORT; }
 
         virtual void finish(
-	  Pathname /*path*/
+          Pathname /*path*/
           , Error /*error*/
-	  , const std::string &/*reason*/
+          , const std::string &/*reason*/
         ) {}
       };
 
@@ -634,27 +647,226 @@ namespace zypp
         enum Action {
           ABORT,  // abort and return error
           RETRY,  // retry
-	  IGNORE  // ignore the failure
+          IGNORE  // ignore the failure
         };
 
         enum Error {
-	  NO_ERROR,
-	  FAILED // conversion failed
+          NO_ERROR,
+          FAILED // conversion failed
         };
 
         virtual void start( Pathname /*path*/ )
-	{}
+        {}
 
         virtual bool progress( int/*value*/, Pathname/*path*/ )
         { return true; }
 
         virtual Action problem( Pathname/*path*/, Error/*error*/, const std::string &/*description*/ )
-	{ return ABORT; }
+        { return ABORT; }
 
         virtual void finish( Pathname/*path*/, Error/*error*/, const std::string &/*reason*/ )
-	{}
+        {}
       };
 #endif
+
+      ///////////////////////////////////////////////////////////////////
+      /// \brief Report active throughout the whole rpm transaction.
+      ///
+      /// The report covers events not directly related to an individual
+      /// sub-task. Mostly log lines.
+      /// \todo maybe include the sub-task reports here as well, so it
+      /// becomes the only report needed to follow the transaction.
+      ///////////////////////////////////////////////////////////////////
+      struct SingleTransReport : public callback::ReportBase
+      {
+        /** \c "zypp-rpm/logline" report a line suitable to be written to the screen.
+         * UserData:
+         * \c "line"  : std::reference_wrapper<const std::string>; the line to show
+         * \c "level" : enum class loglevel; rendering hint
+         */
+        static const UserData::ContentType contentLogline;
+        /** Rendering hint for log-lines to show. */
+        enum class loglevel { dbg, msg, war, err, crt };
+        /** Suggested prefix for log-lines to show. */
+        static const char *const loglevelPrefix( loglevel level_r )
+        {
+          switch ( level_r ) {
+            case loglevel::crt:  return "fatal error: ";
+            case loglevel::err:  return "error: ";
+            case loglevel::war:  return "warning: ";
+            case loglevel::msg:  return "";
+            case loglevel::dbg:  return "D: ";
+          }
+        }
+      };
+
+      // Generic transaction reports, this is used for verifying and preparing tasks, the name param
+      // for the start function defines which report we are looking at
+      struct TransactionReportSA : public callback::ReportBase
+      {
+        enum Error {
+          NO_ERROR, // everything went perfectly fine
+          FINISHED_WITH_ERRORS, // the transaction was finished, but some errors happened
+          FAILED // the transaction failed completely
+        };
+
+        virtual void start(
+          const std::string &/*name*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        virtual void progress(
+          int /*value*/,
+          const UserData & = UserData() /*userdata*/
+          ) { }
+
+        virtual void finish(
+          Error /*error*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+      /** "zypp-rpm/transactionsa": Additional rpm output (sent immediately).
+       * Data:
+       * line     : std::reference_wrapper<const std::string>
+       */
+        static const UserData::ContentType contentRpmout;
+      };
+
+
+      // progress for installing a resolvable in single transaction mode
+      struct InstallResolvableReportSA : public callback::ReportBase
+      {
+        enum Error {
+          NO_ERROR,
+          NOT_FOUND, 	// the requested Url was not found
+          IO,		// IO error
+          INVALID	// th resolvable is invalid
+        };
+
+        virtual void start(
+          Resolvable::constPtr /*resolvable*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        virtual void progress(
+          int /*value*/,
+          Resolvable::constPtr /*resolvable*/,
+          const UserData & = UserData() /*userdata*/
+          ) { return; }
+
+        virtual void finish(
+          Resolvable::constPtr /*resolvable*/
+          , Error /*error*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        /** "zypp-rpm/installpkgsa": Additional rpm output (sent immediately).
+         * Data:
+         * solvable : satSolvable processed
+         * line     : std::reference_wrapper<const std::string>
+         */
+        static const UserData::ContentType contentRpmout;
+      };
+
+      // progress for removing a resolvable in single transaction mode
+      struct RemoveResolvableReportSA : public callback::ReportBase
+      {
+        enum Error {
+          NO_ERROR,
+          NOT_FOUND, 	// the requested Url was not found
+          IO,		// IO error
+          INVALID       // th resolvable is invalid
+        };
+
+        virtual void start(
+          Resolvable::constPtr /*resolvable*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        virtual void progress(
+          int /*value*/,
+          Resolvable::constPtr /*resolvable*/,
+          const UserData & = UserData() /*userdata*/
+          ) { return; }
+
+        virtual void finish(
+          Resolvable::constPtr /*resolvable*/
+          , Error /*error*/
+          , const UserData & = UserData() /*userdata*/
+          ) {}
+
+        /** "zypp-rpm/removepkgsa": Additional rpm output (sent immediately).
+         * For data \see \ref InstallResolvableReportSA::contentRpmout
+         */
+        static const UserData::ContentType contentRpmout;
+      };
+
+      // progress for cleaning up the old version of a package after it was upgraded to a new version
+      struct CleanupPackageReportSA : public callback::ReportBase
+      {
+        enum Error {
+          NO_ERROR
+        };
+
+        virtual void start(
+          const std::string & /*nvra*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        virtual void progress(
+          int /*value*/,
+          const UserData & = UserData() /*userdata*/
+          ) { return; }
+
+        virtual void finish(
+          Error /*error*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        /** "zypp-rpm/cleanupkgsa": Additional rpm output (sent immediately).
+         * For data \see \ref InstallResolvableReportSA::contentRpmout
+         */
+        static const UserData::ContentType contentRpmout;
+      };
+
+
+      // progress for script thats executed during a commit transaction
+      // the resolvable can be null, for things like posttrans scripts
+      struct CommitScriptReportSA : public callback::ReportBase
+      {
+        enum Error {
+          NO_ERROR,
+          WARN,
+          CRITICAL // the script failure prevented solvable installation
+        };
+
+        virtual void start(
+          const std::string &  /*scriptType*/,
+          const std::string &  /*packageName ( can be empty )*/,
+          Resolvable::constPtr /*resolvable  ( can be null )*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        virtual void progress(
+          int /*value*/,
+          Resolvable::constPtr /*resolvable*/,
+          const UserData & = UserData() /*userdata*/
+          ) { return; }
+
+        virtual void finish(
+          Resolvable::constPtr /*resolvable*/
+          , Error /*error*/,
+          const UserData & = UserData() /*userdata*/
+          ) {}
+
+        /** "zypp-rpm/scriptsa": Additional rpm output (sent immediately).
+         * Data:
+         * solvable : satSolvable processed ( can be empty )
+         * line     : std::reference_wrapper<const std::string>
+         */
+        static const UserData::ContentType contentRpmout;
+      };
+
       /////////////////////////////////////////////////////////////////
     } // namespace rpm
     ///////////////////////////////////////////////////////////////////
@@ -747,7 +959,7 @@ namespace zypp
      * type of conflict of old and new lock
      */
     enum ConflictState{
-      SAME_RESULTS, /**< locks lock same item in pool but his parameters is different */
+      SAME_RESULTS, /**< locks lock same item in pool but its parameters are different */
       INTERSECT /**< locks lock some file and unlocking lock unlock only part
       * of iti, so removing old lock can unlock more items in pool */
     };
@@ -765,7 +977,7 @@ namespace zypp
      * When user unlock something which is locked by non-identical query
      */
     virtual Action conflict(
-	 const PoolQuery&, /**< problematic query*/
+         const PoolQuery&, /**< problematic query*/
        ConflictState
      ) { return DELETE; }
 
@@ -827,6 +1039,21 @@ namespace zypp
     //@}
   };
 
+  ///////////////////////////////////////////////////////////////////
+  /// \class UserDataJobReport
+  /// \brief \ref JobReport convenience sending this instance of \ref UserData with each message.
+  ///////////////////////////////////////////////////////////////////
+  struct UserDataJobReport : public JobReport::UserData
+  {
+    using JobReport::UserData::UserData;
+
+    bool debug( const std::string & msg_r )     { return JobReport::debug( msg_r, *this ); }
+    bool info( const std::string & msg_r )      { return JobReport::info( msg_r, *this ); }
+    bool warning( const std::string & msg_r )   { return JobReport::warning( msg_r, *this ); }
+    bool error( const std::string & msg_r )     { return JobReport::error( msg_r, *this ); }
+    bool important( const std::string & msg_r ) { return JobReport::important( msg_r, *this ); }
+    bool data( const std::string & msg_r )      { return JobReport::data( msg_r, *this ); }
+  };
 
   /////////////////////////////////////////////////////////////////
 } // namespace zypp
