@@ -14,17 +14,16 @@
 #include <iostream>
 #include <typeinfo>
 
-#include "zypp/media/MediaException.h"
-#include "zypp/media/MediaManager.h"
-#include "zypp/media/MediaHandler.h"
-#include "zypp/media/Mount.h"
-#include "zypp/thread/Mutex.h"
-#include "zypp/thread/MutexLock.h"
+#include <zypp-media/MediaException>
+#include <zypp/media/MediaManager.h>
+#include <zypp/media/MediaHandlerFactory.h>
+#include <zypp/media/MediaHandler.h>
+#include <zypp-media/Mount>
 
-#include "zypp/base/String.h"
-#include "zypp/base/Logger.h"
-#include "zypp/Pathname.h"
-#include "zypp/PathInfo.h"
+#include <zypp/base/String.h>
+#include <zypp/base/Logger.h>
+#include <zypp/Pathname.h>
+#include <zypp/PathInfo.h>
 
 //////////////////////////////////////////////////////////////////////
 namespace zypp
@@ -34,50 +33,96 @@ namespace zypp
   namespace media
   { //////////////////////////////////////////////////////////////////
 
-    using zypp::thread::Mutex;
-    using zypp::thread::MutexLock;
-
     //////////////////////////////////////////////////////////////////
     namespace // anonymous
     { ////////////////////////////////////////////////////////////////
 
-
-      // -------------------------------------------------------------
-      // STATIC
-      static Mutex  g_Mutex;
-
+      struct ManagedMedia;
+      std::ostream & operator<<( std::ostream & str, const ManagedMedia & obj );
 
       // -------------------------------------------------------------
       struct ManagedMedia
       {
         ~ManagedMedia()
+        {
+          try
+          {
+            if ( _handler )
+              close(); // !!! make sure handler gets properly deleted.
+          }
+          catch(...) {}
+        }
+
+        ManagedMedia( ManagedMedia &&m )
+          : desired ( m.desired )
+          , verifier( std::move(m.verifier) )
+          , _handler ( std::move(m._handler) )
         {}
 
-        ManagedMedia()
-          : desired (false)
-        {}
+        static ManagedMedia makeManagedMedia ( const Url &o_url, const Pathname &preferred_attach_point, const MediaVerifierRef &v )
+        {
+          auto handler = MediaHandlerFactory::createHandler( o_url, preferred_attach_point );
+          if ( !handler ) {
+            ERR << "Failed to create media handler" << std::endl;
+            ZYPP_THROW( MediaSystemException(o_url, "Failed to create media handler"));
+          }
+          return ManagedMedia( std::move(handler), v );
+        }
 
-        ManagedMedia(const ManagedMedia &m)
-          : desired (m.desired)
-          , handler (m.handler)
-          , verifier(m.verifier)
-        {}
+        ManagedMedia &operator= ( ManagedMedia &&other ) = default;
 
-        ManagedMedia(const MediaAccessRef &h, const MediaVerifierRef &v)
-          : desired (false)
-          , handler (h)
-          , verifier(v)
-        {}
+        operator bool () const {
+          return ( _handler ? true : false );
+        }
+
+        inline MediaHandler &handler() {
+          if ( !_handler )
+              ZYPP_THROW(MediaNotOpenException("Accessing ManagedMedia after it was closed"));
+          return *_handler;
+        }
+
+        inline const MediaHandler &handler() const {
+          if ( !_handler )
+            ZYPP_THROW(MediaNotOpenException("Accessing ManagedMedia after it was closed"));
+          return *_handler;
+        }
+
+        std::ostream & dumpOn( std::ostream & str ) const {
+          if ( !_handler )
+            return str << "ManagedMedia( closed )";
+
+          str << _handler->protocol() << "(" << *_handler << ")";
+          return str;
+        }
+
+        inline void close ()
+        {
+          ///////////////////////////////////////////////////////////////////
+          // !!! make shure handler gets properly deleted.
+          // I.e. release attached media before deleting the handler.
+          ///////////////////////////////////////////////////////////////////
+
+          try {
+            handler().release();
+          }
+          catch (const MediaException & excpt_r)
+          {
+            ZYPP_CAUGHT(excpt_r);
+            WAR << "Close: " << *this << " (" << excpt_r << ")" << std::endl;
+            ZYPP_RETHROW(excpt_r);
+          }
+          MIL << "Close: " << *this << " (OK)" << std::endl;
+        }
 
         inline void
         checkAttached(MediaAccessId id)
         {
-          if( !handler->isAttached())
+          if( !handler().isAttached())
           {
             DBG << "checkAttached(" << id << ") not attached" << std::endl;
             desired = false;
             ZYPP_THROW(MediaNotAttachedException(
-              handler->url()
+              handler().url()
             ));
           }
         }
@@ -88,12 +133,13 @@ namespace zypp
 
           if ( !desired )
           {
+            const auto &hdl = handler();
             try {
-              desired = verifier->isDesiredMedia(handler);
+              desired = verifier->isDesiredMedia( handler() );
             } catch ( const zypp::Exception &e ) {
                 ZYPP_CAUGHT( e );
 
-                media::MediaNotDesiredException newEx ( handler->url() );
+                media::MediaNotDesiredException newEx ( hdl.url() );
                 newEx.remember( e );
                 ZYPP_THROW( newEx );
             }
@@ -101,7 +147,7 @@ namespace zypp
             if( !desired )
             {
               DBG << "checkDesired(" << id << "): not desired (report by " << verifier->info() << ")" << std::endl;
-              ZYPP_THROW( MediaNotDesiredException( handler->url() ) );
+              ZYPP_THROW( MediaNotDesiredException( hdl.url() ) );
             }
 
             DBG << "checkDesired(" << id << "): desired (report by " << verifier->info() << ")" << std::endl;
@@ -111,10 +157,22 @@ namespace zypp
         }
 
         bool             desired;
-        MediaAccessRef   handler;
         MediaVerifierRef verifier;
+        Pathname         deltafile;
+
+      private:
+        ManagedMedia( std::unique_ptr<MediaHandler> &&h, const MediaVerifierRef &v)
+          : desired (false)
+          , verifier(v)
+          , _handler ( std::move(h) )
+        {}
+
+        std::unique_ptr<MediaHandler>  _handler;
       };
 
+      std::ostream & operator<<( std::ostream & str, const ManagedMedia & obj ) {
+        return obj.dumpOn( str );
+      }
 
       // -------------------------------------------------------------
       typedef std::map<MediaAccessId, ManagedMedia> ManagedMediaMap;
@@ -156,8 +214,6 @@ namespace zypp
     public:
       ~MediaManager_Impl()
       {
-        MutexLock glock(g_Mutex);
-
         try
         {
           // remove depending (iso) handlers first
@@ -168,13 +224,13 @@ namespace zypp
             found = false;
             for(it = mediaMap.begin(); it != mediaMap.end(); /**/)
             {
-              if( it->second.handler->dependsOnParent())
+              if( it->second && it->second.handler().dependsOnParent() )
               {
                 found = true;
                 // let it forget its parent, we will
                 // destroy it later (in clear())...
-                it->second.handler->resetParentId();
-                mediaMap.erase( it++ ); // postfix! Incrementing before erase
+                it->second.handler().resetParentId();
+                it = mediaMap.erase( it ); // postfix! Incrementing before erase
               } else {
                 ++it;
               }
@@ -242,7 +298,6 @@ namespace zypp
     //////////////////////////////////////////////////////////////////
     MediaManager::MediaManager()
     {
-      MutexLock glock(g_Mutex);
       if( !m_impl)
       {
         m_impl.reset( new MediaManager_Impl());
@@ -258,18 +313,14 @@ namespace zypp
     MediaAccessId
     MediaManager::open(const Url &url, const Pathname &preferred_attach_point)
     {
-      MutexLock glock(g_Mutex);
-
       // create new access handler for it
-      MediaAccessRef handler( new MediaAccess());
       MediaVerifierRef verifier( new NoVerifier());
-      ManagedMedia tmp( handler, verifier);
-
-      tmp.handler->open(url, preferred_attach_point);
+      ManagedMedia tmp = ManagedMedia::makeManagedMedia( url, preferred_attach_point, verifier );
 
       MediaAccessId nextId = m_impl->nextAccessId();
 
-      m_impl->mediaMap[nextId] = tmp;
+      m_impl->mediaMap.insert( std::make_pair( nextId, std::move(tmp) ) );
+      //m_impl->mediaMap[nextId] = std::move(tmp);
 
       DBG << "Opened new media access using id " << nextId
           << " to " << url.asString() << std::endl;
@@ -280,8 +331,6 @@ namespace zypp
     void
     MediaManager::close(MediaAccessId accessId)
     {
-      MutexLock glock(g_Mutex);
-
       //
       // The MediaISO handler internally requests an accessId
       // of a "parent" handler providing the iso file.
@@ -295,10 +344,10 @@ namespace zypp
       ManagedMediaMap::iterator m(m_impl->mediaMap.begin());
       for( ; m != m_impl->mediaMap.end(); ++m)
       {
-        if( m->second.handler->dependsOnParent(accessId, true))
+        if( m->second.handler().dependsOnParent(accessId, true))
         {
           ZYPP_THROW(MediaIsSharedException(
-            m->second.handler->url().asString()
+            m->second.handler().url().asString()
           ));
         }
       }
@@ -307,7 +356,7 @@ namespace zypp
           << accessId << " requested" << std::endl;
 
       ManagedMedia &ref( m_impl->findMM(accessId));
-      ref.handler->close();
+      ref.close();
 
       m_impl->mediaMap.erase(accessId);
     }
@@ -316,44 +365,35 @@ namespace zypp
     bool
     MediaManager::isOpen(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMediaMap::iterator it( m_impl->mediaMap.find(accessId));
-      return it != m_impl->mediaMap.end() &&
-             it->second.handler->isOpen();
+      return it != m_impl->mediaMap.end();
     }
 
     // ---------------------------------------------------------------
     std::string
     MediaManager::protocol(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->protocol();
+      return ref.handler().protocol();
     }
 
     // ---------------------------------------------------------------
-	  bool
+          bool
     MediaManager::downloads(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->downloads();
+      return ref.handler().downloads();
     }
 
     // ---------------------------------------------------------------
     Url
     MediaManager::url(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->url();
+      return ref.handler().url();
     }
 
     // ---------------------------------------------------------------
@@ -361,8 +401,6 @@ namespace zypp
     MediaManager::addVerifier(MediaAccessId           accessId,
                               const MediaVerifierRef &verifier)
     {
-      MutexLock glock(g_Mutex);
-
       if( !verifier)
         ZYPP_THROW(MediaException("Invalid verifier reference"));
 
@@ -379,8 +417,6 @@ namespace zypp
     void
     MediaManager::delVerifier(MediaAccessId accessId)
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       MediaVerifierRef verifier( new NoVerifier());
@@ -395,22 +431,19 @@ namespace zypp
     bool
     MediaManager::setAttachPrefix(const Pathname &attach_prefix)
     {
-      MutexLock glock(g_Mutex);
-
       return MediaHandler::setAttachPrefix(attach_prefix);
     }
 
     // ---------------------------------------------------------------
     void MediaManager::attach(MediaAccessId accessId)
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
+      auto &hdl = ref.handler();
 
       DBG << "attach(id=" << accessId << ")" << std::endl;
 
       // try first mountable/mounted device
-      ref.handler->attach(false);
+      hdl.attach(false);
       try
       {
         ref.checkDesired(accessId);
@@ -420,22 +453,22 @@ namespace zypp
       {
         ZYPP_CAUGHT(ex);
 
-        if (!ref.handler->hasMoreDevices())
+        if (!hdl.hasMoreDevices())
           ZYPP_RETHROW(ex);
 
-        if (ref.handler->isAttached())
-          ref.handler->release();
+        if (hdl.isAttached())
+          hdl.release();
       }
 
       MIL << "checkDesired(" << accessId << ") of first device failed,"
         " going to try others with attach(true)" << std::endl;
 
-      while (ref.handler->hasMoreDevices())
+      while (hdl.hasMoreDevices())
       {
         try
         {
           // try to attach next device
-          ref.handler->attach(true);
+          hdl.attach(true);
           ref.checkDesired(accessId);
           return;
         }
@@ -443,28 +476,28 @@ namespace zypp
         {
           ZYPP_CAUGHT(ex);
 
-          if (!ref.handler->hasMoreDevices())
+          if (!hdl.hasMoreDevices())
           {
             MIL << "No desired media found after trying all detected devices." << std::endl;
             ZYPP_RETHROW(ex);
           }
 
-          AttachedMedia media(ref.handler->attachedMedia());
+          AttachedMedia media(hdl.attachedMedia());
           DBG << "Skipping " << media.mediaSource->asString() << ": not desired media." << std::endl;
 
-          ref.handler->release();
+          hdl.release();
         }
         catch (const MediaException & ex)
         {
           ZYPP_CAUGHT(ex);
 
-          if (!ref.handler->hasMoreDevices())
+          if (!hdl.hasMoreDevices())
             ZYPP_RETHROW(ex);
 
-          AttachedMedia media(ref.handler->attachedMedia());
+          AttachedMedia media(hdl.attachedMedia());
           DBG << "Skipping " << media.mediaSource->asString() << " because of exception thrown by attach(true)" << std::endl;
 
-          if (ref.handler->isAttached()) ref.handler->release();
+          if (hdl.isAttached()) hdl.release();
         }
       }
     }
@@ -473,8 +506,6 @@ namespace zypp
     void
     MediaManager::release(MediaAccessId accessId, const std::string & ejectDev)
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       DBG << "release(id=" << accessId;
@@ -493,14 +524,15 @@ namespace zypp
         ManagedMediaMap::iterator m(m_impl->mediaMap.begin());
         for( ; m != m_impl->mediaMap.end(); ++m)
         {
-          if( m->second.handler->dependsOnParent(accessId, false))
+          auto &hdl = m->second.handler();
+          if( hdl.dependsOnParent(accessId, false))
           {
             try
             {
               DBG << "Forcing release of handler depending on access id "
                   << accessId << std::endl;
               m->second.desired  = false;
-              m->second.handler->release();
+              hdl.release();
             }
             catch(const MediaException &e)
             {
@@ -510,30 +542,29 @@ namespace zypp
         }
       }
       ref.desired  = false;
-      ref.handler->release(ejectDev);
+      ref.handler().release(ejectDev);
     }
 
     // ---------------------------------------------------------------
     void
     MediaManager::releaseAll()
     {
-      MutexLock glock(g_Mutex);
-
       MIL << "Releasing all attached media" << std::endl;
 
       ManagedMediaMap::iterator m(m_impl->mediaMap.begin());
       for( ; m != m_impl->mediaMap.end(); ++m)
       {
-        if( m->second.handler->dependsOnParent())
+        auto &hdl = m->second.handler();
+        if( hdl.dependsOnParent())
           continue;
 
         try
         {
-          if(m->second.handler->isAttached())
+          if(hdl.isAttached())
           {
             DBG << "Releasing media id " << m->first << std::endl;
             m->second.desired  = false;
-            m->second.handler->release();
+            hdl.release();
           }
           else
           {
@@ -554,50 +585,42 @@ namespace zypp
     void
     MediaManager::disconnect(MediaAccessId accessId)
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      ref.handler->disconnect();
+      ref.handler().disconnect();
     }
 
     // ---------------------------------------------------------------
     bool
     MediaManager::isAttached(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->isAttached();
+      return ref.handler().isAttached();
     }
 
     // ---------------------------------------------------------------
     bool MediaManager::isSharedMedia(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->isSharedMedia();
+      return ref.handler().isSharedMedia();
     }
 
     // ---------------------------------------------------------------
     bool
     MediaManager::isDesiredMedia(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      if( !ref.handler->isAttached())
+      if( !ref.handler().isAttached())
       {
         ref.desired = false;
       }
       else
       {
         try {
-          ref.desired = ref.verifier->isDesiredMedia(ref.handler);
+          ref.desired = ref.verifier->isDesiredMedia( ref.handler() );
         }
         catch(const zypp::Exception &e) {
           ZYPP_CAUGHT(e);
@@ -616,8 +639,6 @@ namespace zypp
     MediaManager::isDesiredMedia(MediaAccessId           accessId,
                                  const MediaVerifierRef &verifier) const
     {
-      MutexLock glock(g_Mutex);
-
       MediaVerifierRef v(verifier);
       if( !v)
         ZYPP_THROW(MediaException("Invalid verifier reference"));
@@ -625,10 +646,10 @@ namespace zypp
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       bool desired = false;
-      if( ref.handler->isAttached())
+      if( ref.handler().isAttached())
       {
         try {
-          desired = v->isDesiredMedia(ref.handler);
+          desired = v->isDesiredMedia( ref.handler() );
         }
         catch(const zypp::Exception &e) {
           ZYPP_CAUGHT(e);
@@ -653,12 +674,10 @@ namespace zypp
     Pathname
     MediaManager::localRoot(MediaAccessId accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       Pathname path;
-      path = ref.handler->localRoot();
+      path = ref.handler().localRoot();
       return path;
     }
 
@@ -667,12 +686,10 @@ namespace zypp
     MediaManager::localPath(MediaAccessId accessId,
                             const Pathname & pathname) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       Pathname path;
-      path = ref.handler->localPath(pathname);
+      path = ref.handler().localPath(pathname);
       return path;
     }
 
@@ -681,13 +698,13 @@ namespace zypp
                               const Pathname &filename,
                               const ByteCount &expectedFileSize ) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      ref.checkDesired(accessId);
+      auto loc = OnMediaLocation( filename )
+                   .setDownloadSize( expectedFileSize )
+                   .setDeltafile( ref.deltafile );
 
-      ref.handler->provideFile(filename, expectedFileSize);
+      provideFile( accessId, loc );
     }
 
     // ---------------------------------------------------------------
@@ -695,7 +712,21 @@ namespace zypp
     MediaManager::provideFile(MediaAccessId   accessId,
                               const Pathname &filename ) const
     {
-      provideFile( accessId, filename, 0);
+      ManagedMedia &ref( m_impl->findMM(accessId));
+
+      auto loc = OnMediaLocation( filename )
+                   .setDeltafile( ref.deltafile );
+
+      provideFile( accessId, loc );
+    }
+
+    void MediaManager::provideFile( MediaAccessId accessId, const OnMediaLocation &file ) const
+    {
+      ManagedMedia &ref( m_impl->findMM(accessId));
+
+      ref.checkDesired(accessId);
+
+      ref.handler().provideFile( file );
     }
 
     // ---------------------------------------------------------------
@@ -703,13 +734,20 @@ namespace zypp
     MediaManager::setDeltafile(MediaAccessId   accessId,
                               const Pathname &filename ) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkDesired(accessId);
 
-      ref.handler->setDeltafile(filename);
+      ref.deltafile = filename;
+    }
+
+    void MediaManager::precacheFiles(MediaAccessId accessId, const std::vector<OnMediaLocation> &files)
+    {
+      ManagedMedia &ref( m_impl->findMM(accessId));
+
+      ref.checkDesired(accessId);
+
+      ref.handler().precacheFiles( files );
     }
 
     // ---------------------------------------------------------------
@@ -717,13 +755,11 @@ namespace zypp
     MediaManager::provideDir(MediaAccessId   accessId,
                              const Pathname &dirname) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkDesired(accessId);
 
-      ref.handler->provideDir(dirname);
+      ref.handler().provideDir(dirname);
     }
 
     // ---------------------------------------------------------------
@@ -731,13 +767,11 @@ namespace zypp
     MediaManager::provideDirTree(MediaAccessId   accessId,
                                  const Pathname &dirname) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkDesired(accessId);
 
-      ref.handler->provideDirTree(dirname);
+      ref.handler().provideDirTree(dirname);
     }
 
     // ---------------------------------------------------------------
@@ -745,13 +779,11 @@ namespace zypp
     MediaManager::releaseFile(MediaAccessId   accessId,
                               const Pathname &filename) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkAttached(accessId);
 
-      ref.handler->releaseFile(filename);
+      ref.handler().releaseFile(filename);
     }
 
     // ---------------------------------------------------------------
@@ -759,13 +791,11 @@ namespace zypp
     MediaManager::releaseDir(MediaAccessId   accessId,
                              const Pathname &dirname) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkAttached(accessId);
 
-      ref.handler->releaseDir(dirname);
+      ref.handler().releaseDir(dirname);
     }
 
 
@@ -774,13 +804,11 @@ namespace zypp
     MediaManager::releasePath(MediaAccessId   accessId,
                               const Pathname &pathname) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       ref.checkAttached(accessId);
 
-      ref.handler->releasePath(pathname);
+      ref.handler().releasePath(pathname);
     }
 
     // ---------------------------------------------------------------
@@ -790,14 +818,12 @@ namespace zypp
                           const Pathname         &dirname,
                           bool                    dots) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       // FIXME: ref.checkDesired(accessId); ???
       ref.checkAttached(accessId);
 
-      ref.handler->dirInfo(retlist, dirname, dots);
+      ref.handler().dirInfo(retlist, dirname, dots);
     }
 
     // ---------------------------------------------------------------
@@ -807,27 +833,24 @@ namespace zypp
                           const Pathname         &dirname,
                           bool                    dots) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       // FIXME: ref.checkDesired(accessId); ???
       ref.checkAttached(accessId);
 
-      ref.handler->dirInfo(retlist, dirname, dots);
+      ref.handler().dirInfo(retlist, dirname, dots);
     }
 
     // ---------------------------------------------------------------
     bool
     MediaManager::doesFileExist(MediaAccessId  accessId, const Pathname & filename ) const
     {
-      MutexLock glock(g_Mutex);
       ManagedMedia &ref( m_impl->findMM(accessId));
 
       // FIXME: ref.checkDesired(accessId); ???
       ref.checkAttached(accessId);
 
-      return ref.handler->doesFileExist(filename);
+      return ref.handler().doesFileExist(filename);
     }
 
     // ---------------------------------------------------------------
@@ -836,9 +859,8 @@ namespace zypp
                                      std::vector<std::string> & devices,
                                      unsigned int & index) const
     {
-      MutexLock glock(g_Mutex);
       ManagedMedia &ref( m_impl->findMM(accessId));
-      return ref.handler->getDetectedDevices(devices, index);
+      return ref.handler().getDetectedDevices(devices, index);
     }
 
     // ---------------------------------------------------------------
@@ -846,7 +868,6 @@ namespace zypp
     time_t
     MediaManager::getMountTableMTime()
     {
-      MutexLock glock(g_Mutex);
       return MediaManager_Impl::getMountTableMTime();
     }
 
@@ -855,8 +876,6 @@ namespace zypp
     MountEntries
     MediaManager::getMountEntries()
     {
-      MutexLock glock(g_Mutex);
-
       return MediaManager_Impl::getMountEntries();
     }
 
@@ -868,15 +887,13 @@ namespace zypp
       if( path.empty() || path == "/" || !PathInfo(path).isDir())
         return false;
 
-      MutexLock glock(g_Mutex);
-
       //
       // check against our current attach points
       //
       ManagedMediaMap::const_iterator m(m_impl->mediaMap.begin());
       for( ; m != m_impl->mediaMap.end(); ++m)
       {
-        AttachedMedia ret = m->second.handler->attachedMedia();
+        AttachedMedia ret = m->second.handler().attachedMedia();
         if( ret.mediaSource && ret.attachPoint)
         {
           std::string mnt(ret.attachPoint->path.asString());
@@ -935,29 +952,25 @@ namespace zypp
     AttachedMedia
     MediaManager::getAttachedMedia(MediaAccessId &accessId) const
     {
-      MutexLock glock(g_Mutex);
-
       ManagedMedia &ref( m_impl->findMM(accessId));
 
-      return ref.handler->attachedMedia();
+      return ref.handler().attachedMedia();
     }
 
     // ---------------------------------------------------------------
     AttachedMedia
     MediaManager::findAttachedMedia(const MediaSourceRef &media) const
     {
-      MutexLock glock(g_Mutex);
-
       if( !media || media->type.empty())
         return AttachedMedia();
 
       ManagedMediaMap::const_iterator m(m_impl->mediaMap.begin());
       for( ; m != m_impl->mediaMap.end(); ++m)
       {
-        if( !m->second.handler->isAttached())
+        if( !m->second.handler().isAttached())
           continue;
 
-        AttachedMedia ret = m->second.handler->attachedMedia();
+        AttachedMedia ret = m->second.handler().attachedMedia();
         if( ret.mediaSource && ret.mediaSource->equals( *media))
             return ret;
       }
@@ -968,21 +981,19 @@ namespace zypp
     void
     MediaManager::forceReleaseShared(const MediaSourceRef &media)
     {
-      MutexLock glock(g_Mutex);
-
       if( !media || media->type.empty())
         return;
 
       ManagedMediaMap::iterator m(m_impl->mediaMap.begin());
       for( ; m != m_impl->mediaMap.end(); ++m)
       {
-        if( !m->second.handler->isAttached())
+        if( !m->second.handler().isAttached())
           continue;
 
-        AttachedMedia ret = m->second.handler->attachedMedia();
+        AttachedMedia ret = m->second.handler().attachedMedia();
         if( ret.mediaSource && ret.mediaSource->equals( *media))
         {
-          m->second.handler->release();
+          m->second.handler().release();
           m->second.desired  = false;
         }
       }
