@@ -14,6 +14,7 @@
 #include <unordered_set>
 #include <iterator>
 #include <stdio.h>
+#include <zypp/base/LogControl.h>
 #include <zypp/base/LogTools.h>
 #include <zypp/base/String.h>
 #include <zypp/base/Gettext.h>
@@ -23,7 +24,7 @@
 #include <zypp/ExternalProgram.h>
 #include <zypp/base/Regex.h>
 #include <zypp/base/IOStream.h>
-#include <zypp/base/InputStream.h>
+#include <zypp-core/base/InputStream>
 #include <zypp/target/rpm/librpmDb.h>
 
 #include <zypp/misc/CheckAccessDeleted.h>
@@ -174,13 +175,13 @@ namespace zypp
       // RpmDb access is blocked while the Target is not initialized.
       // Launching the Target just for this query would be an overkill.
       struct TmpUnblock {
-	TmpUnblock()
-	: _wasBlocked( librpmDb::isBlocked() )
-	{ if ( _wasBlocked ) librpmDb::unblockAccess(); }
-	~TmpUnblock()
-	{ if ( _wasBlocked ) librpmDb::blockAccess(); }
+        TmpUnblock()
+        : _wasBlocked( librpmDb::isBlocked() )
+        { if ( _wasBlocked ) librpmDb::unblockAccess(); }
+        ~TmpUnblock()
+        { if ( _wasBlocked ) librpmDb::blockAccess(); }
       private:
-	bool _wasBlocked;
+        bool _wasBlocked;
       } tmpUnblock;
 
       librpmDb::db_const_iterator it;
@@ -349,7 +350,7 @@ namespace zypp
         , "/dev/"
         , "/tmp/"
         , "/proc/"
-	, "/memfd:"
+        , "/memfd:"
       };
       for_( it, arrayBegin( black ), arrayEnd( black ) )
       {
@@ -396,6 +397,8 @@ namespace zypp
 
     pid_t cachepid = 0;
     FilterRunsInContainer runsInLXC;
+    MIL << "Silently scanning lsof output..." << endl;
+    zypp::base::LogControl::TmpLineWriter shutUp;	// suppress excessive readdir etc. logging in runsInLXC
     for( std::string line = source.receiveLine( 30 * 1000 ); ! line.empty(); line = source.receiveLine(  30 * 1000  ) )
     {
       // NOTE: line contains '\0' separeated fields!
@@ -440,7 +443,7 @@ namespace zypp
       cachemap = _pimpl->filterInput( prog );
     } catch ( const io::TimeoutException &e ) {
       ZYPP_CAUGHT( e );
-      prog.close();
+      prog.kill();
       ZYPP_THROW ( Exception( "Reading data from 'lsof' timed out.") );
     }
 
@@ -529,19 +532,19 @@ namespace zypp
 
   std::string CheckAccessDeleted::ProcInfo::service() const
   {
-    static const str::regex rx( "[0-9]+:name=systemd:/system.slice/(.*/)?(.*).service$" );
+    static const str::regex rx( "(0::|[0-9]+:name=systemd:)/system.slice/(.*/)?(.*).service$" );
     str::smatch what;
     std::string ret;
     iostr::simpleParseFile( InputStream( Pathname("/proc")/pid/"cgroup" ),
-			    [&]( int num_r, std::string line_r )->bool
-			    {
-			      if ( str::regex_match( line_r, what, rx ) )
-			      {
-				ret = what[2];
-				return false;	// stop after match
-			      }
-			      return true;
-			    } );
+                            [&]( int num_r, std::string line_r )->bool
+                            {
+                              if ( str::regex_match( line_r, what, rx ) )
+                              {
+                                ret = what[3];
+                                return false;	// stop after match
+                              }
+                              return true;
+                            } );
     return ret;
   }
 
