@@ -11,23 +11,23 @@
 */
 #include <iostream>
 
-#include "zypp/base/Logger.h"
-#include "zypp/base/Gettext.h"
-#include "zypp/base/Exception.h"
-#include "zypp/base/Functional.h"
-#include "zypp/base/Collector.h"
-#include "zypp/base/Xml.h"
+#include <zypp/base/Logger.h>
+#include <zypp/base/Gettext.h>
+#include <zypp/base/Exception.h>
+#include <zypp/base/Functional.h>
+#include <zypp/base/Collector.h>
+#include <zypp/base/Xml.h>
 
-#include "zypp/sat/detail/PoolImpl.h"
-#include "zypp/sat/Solvable.h"
-#include "zypp/sat/Pool.h"
-#include "zypp/sat/LookupAttr.h"
+#include <zypp/sat/detail/PoolImpl.h>
+#include <zypp/sat/Solvable.h>
+#include <zypp/sat/Pool.h>
+#include <zypp/sat/LookupAttr.h>
 
-#include "zypp/Repository.h"
-#include "zypp/OnMediaLocation.h"
-#include "zypp/ZConfig.h"
+#include <zypp/Repository.h>
+#include <zypp-core/OnMediaLocation>
+#include <zypp/ZConfig.h>
 
-#include "zypp/ui/Selectable.h"
+#include <zypp/ui/Selectable.h>
 
 using std::endl;
 
@@ -45,26 +45,26 @@ namespace zypp
         if ( ! _ident )
           return;
 
-	ResKind explicitKind = ResKind::explicitBuiltin( _ident.c_str() );
-	// NOTE: kind package and srcpackage do not have namespaced ident!
-	if ( ! explicitKind  )
-	{
+        ResKind explicitKind = ResKind::explicitBuiltin( _ident.c_str() );
+        // NOTE: kind package and srcpackage do not have namespaced ident!
+        if ( ! explicitKind  )
+        {
           _name = _ident;
-	  // No kind defaults to package
-	  if ( !_kind )
-	    _kind = ResKind::package;
-	  else if ( ! ( _kind == ResKind::package || _kind == ResKind::srcpackage ) )
-	    _ident = IdString( str::form( "%s:%s", _kind.c_str(), _ident.c_str() ) );
-	}
-	else
-	{
-	  // strip kind spec from name
-	  _name = IdString( ::strchr( _ident.c_str(), ':' )+1 );
-	  _kind = explicitKind;
-	  if ( _kind == ResKind::package || _kind == ResKind::srcpackage )
-	    _ident = _name;
-	}
-	return;
+          // No kind defaults to package
+          if ( !_kind )
+            _kind = ResKind::package;
+          else if ( ! ( _kind == ResKind::package || _kind == ResKind::srcpackage ) )
+            _ident = IdString( str::form( "%s:%s", _kind.c_str(), _ident.c_str() ) );
+        }
+        else
+        {
+          // strip kind spec from name
+          _name = IdString( ::strchr( _ident.c_str(), ':' )+1 );
+          _kind = explicitKind;
+          if ( _kind == ResKind::package || _kind == ResKind::srcpackage )
+            _ident = _name;
+        }
+        return;
       }
     } // namespace
     ///////////////////////////////////////////////////////////////////
@@ -97,6 +97,10 @@ namespace zypp
 
     const Solvable Solvable::noSolvable;
 
+    const IdString Solvable::retractedToken	{ "retracted-patch-package()" };
+    const IdString Solvable::ptfMasterToken	{ "ptf()" };
+    const IdString Solvable::ptfPackageToken	{ "ptf-package()" };
+
     /////////////////////////////////////////////////////////////////
 
     detail::CSolvable * Solvable::get() const
@@ -140,13 +144,13 @@ namespace zypp
       }
       else
       {
-	for ( Locale l( lang_r ); l; l = l.fallback() )
-	{
-	  if ( (s = ::solvable_lookup_str_lang( _solvable, attr.id(), l.c_str(), 0 )) )
-	    return s;
-	}
-	// here: no matching locale, so use default
-	s = ::solvable_lookup_str_lang( _solvable, attr.id(), 0, 0 );
+        for ( Locale l( lang_r ); l; l = l.fallback() )
+        {
+          if ( (s = ::solvable_lookup_str_lang( _solvable, attr.id(), l.c_str(), 0 )) )
+            return s;
+        }
+        // here: no matching locale, so use default
+        s = ::solvable_lookup_str_lang( _solvable, attr.id(), 0, 0 );
       }
       return s ? s : std::string();
    }
@@ -226,7 +230,7 @@ namespace zypp
       if ( ! file )
         return OnMediaLocation();
       if ( ! medianr )
-	medianr = 1;
+        medianr = 1;
 
       OnMediaLocation ret;
 
@@ -284,12 +288,12 @@ namespace zypp
       const char * ident = IdString( _solvable->name ).c_str();
       ResKind knownKind( ResKind::explicitBuiltin( ident ) );
       if ( knownKind )
-	return knownKind;
+        return knownKind;
 
       // ...or no ':' in package names (hopefully)...
       const char * sep = ::strchr( ident, ':' );
       if ( ! sep )
-	return ResKind::package;
+        return ResKind::package;
 
       // ...or something unknown.
       return ResKind( std::string( ident, sep-ident ) );
@@ -393,6 +397,36 @@ namespace zypp
       return myPool().isNeedreboot( *this );
     }
 
+    // TODO: Optimize
+    bool Solvable::isBlacklisted() const
+    { return isPtf() || isRetracted(); }
+
+    bool Solvable::isRetracted() const
+    {
+      NO_SOLVABLE_RETURN( false );
+      if ( isKind<Package>() )
+        return myPool().isRetracted( *this );
+      if ( isKind<Patch>() )
+        return lookupStrAttribute( SolvAttr::updateStatus ) == "retracted";
+      return false;
+    }
+
+    bool Solvable::isPtf() const
+    { return isPtfPackage() || isPtfMaster(); }
+
+    bool Solvable::isPtfMaster() const
+    {
+      NO_SOLVABLE_RETURN( false );
+      return myPool().isPtfMaster( *this );
+    }
+
+    bool Solvable::isPtfPackage() const
+    {
+      NO_SOLVABLE_RETURN( false );
+      return myPool().isPtfPackage( *this );
+    }
+
+
     bool Solvable::multiversionInstall() const
     {
       NO_SOLVABLE_RETURN( false );
@@ -442,7 +476,7 @@ namespace zypp
     {
       inline Capabilities _getCapabilities( detail::IdType * idarraydata_r, ::Offset offs_r )
       {
-	return offs_r ? Capabilities( idarraydata_r + offs_r ) : Capabilities();
+        return offs_r ? Capabilities( idarraydata_r + offs_r ) : Capabilities();
       }
     } // namespace
     ///////////////////////////////////////////////////////////////////
@@ -528,6 +562,18 @@ namespace zypp
       return ret;
     }
 
+    std::pair<bool, CapabilitySet> Solvable::matchesSolvable(const SolvAttr &attr, const Solvable &solv) const
+    {
+      sat::Queue capQueue;
+      int res = solvable_matchessolvable( get(), attr.id(), static_cast<Id>( solv.id() ), capQueue, 0 );
+
+      CapabilitySet caps;
+      if ( capQueue.size() )
+        std::for_each( capQueue.begin(), capQueue.end(), [ &caps ]( auto cap ){ caps.insert( Capability(cap) );});
+
+      return std::make_pair( res == 1, std::move(caps) );
+    }
+
     ///////////////////////////////////////////////////////////////////
     namespace
     {
@@ -650,7 +696,7 @@ namespace zypp
       if ( ! file )
         medianr = 0U;
       else if ( ! medianr )
-	medianr = 1U;
+        medianr = 1U;
       return medianr;
     }
 
@@ -702,13 +748,13 @@ namespace zypp
       std::string ret = lookupStrAttribute( SolvAttr::eula, lang_r );
       if ( ret.empty() && isKind<Product>() )
       {
-	const RepoInfo & ri( repoInfo() );
-	std::string riname( name() );	// "license-"+name with fallback "license"
-	if ( ! ri.hasLicense( riname ) )
-	  riname.clear();
+        const RepoInfo & ri( repoInfo() );
+        std::string riname( name() );	// "license-"+name with fallback "license"
+        if ( ! ri.hasLicense( riname ) )
+          riname.clear();
 
-	if ( ri.needToAcceptLicense( riname ) || ! ui::Selectable::get( *this )->hasInstalledObj() )
-	  ret = ri.getLicense( riname, lang_r ); // bnc#908976: suppress informal license upon update
+        if ( ri.needToAcceptLicense( riname ) || ! ui::Selectable::get( *this )->hasInstalledObj() )
+          ret = ri.getLicense( riname, lang_r ); // bnc#908976: suppress informal license upon update
       }
       return ret;
     }
@@ -718,12 +764,12 @@ namespace zypp
       NO_SOLVABLE_RETURN( false );
       if ( isKind<Product>() )
       {
-	const RepoInfo & ri( repoInfo() );
-	std::string riname( name() );	// "license-"+name with fallback "license"
-	if ( ! ri.hasLicense( riname ) )
-	  riname.clear();
+        const RepoInfo & ri( repoInfo() );
+        std::string riname( name() );	// "license-"+name with fallback "license"
+        if ( ! ri.hasLicense( riname ) )
+          riname.clear();
 
-	return ri.needToAcceptLicense( riname );
+        return ri.needToAcceptLicense( riname );
       }
       return true;
     }
