@@ -14,16 +14,16 @@
 
 #include <iosfwd>
 
-#include "zypp/sat/detail/PoolMember.h"
-#include "zypp/sat/SolvAttr.h"
-#include "zypp/ResTraits.h"
-#include "zypp/IdString.h"
-#include "zypp/Edition.h"
-#include "zypp/Arch.h"
-#include "zypp/Dep.h"
-#include "zypp/Capabilities.h"
-#include "zypp/Capability.h"
-#include "zypp/Locale.h"
+#include <zypp/sat/detail/PoolMember.h>
+#include <zypp/sat/SolvAttr.h>
+#include <zypp/ResTraits.h>
+#include <zypp/IdString.h>
+#include <zypp/Edition.h>
+#include <zypp/Arch.h>
+#include <zypp/Dep.h>
+#include <zypp/Capabilities.h>
+#include <zypp/Capability.h>
+#include <zypp/Locale.h>
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -55,6 +55,10 @@ namespace zypp
     public:
       typedef sat::detail::SolvableIdType IdType;
 
+      static const IdString retractedToken;	///< Indicator provides `retracted-patch-package()`
+      static const IdString ptfMasterToken;	///< Indicator provides `ptf()`
+      static const IdString ptfPackageToken;	///< Indicator provides `ptf-package()`
+
     public:
       /** Default ctor creates \ref noSolvable.*/
       Solvable()
@@ -136,14 +140,48 @@ namespace zypp
       /** \overload static version */
       static bool identIsAutoInstalled( const IdString & ident_r );
 
-      /** Whether this solvable triggers the reboot-needed hint if installed/updated. */
-      bool isNeedreboot() const;
-
       /** Whether different versions of this package can be installed at the same time.
        * Per default \c false. \see also \ref ZConfig::multiversion.
        */
       bool multiversionInstall() const;
 
+      /** Whether this solvable triggers the reboot-needed hint if installed/updated. */
+      bool isNeedreboot() const;
+
+      /** \name Blacklisted packages.
+       *
+       * Blacklisted packages are visible in a repository, but no matter how good they
+       * apparently look like, they must not be considered during normal operations.
+       *
+       * Under normal conditions the resolver will avoid selecting blacklisted packages.
+       * To install a blacklisted package it must be selected explicitly. Whether this
+       * is a good idea, depends on the reason for having been blacklisted:
+       *
+       * - RETRACTED packages are usually buggy packages/patches which have been released
+       * and can not be removed from the public repositories anymore. Nevertheless those
+       * packages should not be installed.
+       *
+       * - PTFs usually contain special features or fixes which are not yet released as
+       * an official patch upadate. Installing a PTF will prevent it's packages from being
+       * updated until you decide to remove the PTF and follow the official releases again.
+       */
+      //@{
+      /** Whether this solvable is blacklisted (retracted,ptf,...). */
+      bool isBlacklisted() const;
+
+      /** Whether this solvable is retracted (provides \ref retractedToken). */
+      bool isRetracted() const;
+
+      /** Whether this solvable belongs to a PTF (provides \ref ptfMasterToken or \ref ptfPackageToken). */
+      bool isPtf() const;
+
+      /** Subset of \ref isPtf (provides \ref ptfMasterToken). */
+      bool isPtfMaster() const;
+
+      /** Subset of \ref isPtf (provides \ref ptfPackageToken). */
+      bool isPtfPackage() const;
+      //@}
+
       /** The items build time. */
       Date buildtime() const;
 
@@ -191,19 +229,19 @@ namespace zypp
       /** Return \ref Capabilities selected by \ref Dep constant. */
       Capabilities dep( Dep which_r ) const
       {
-	switch( which_r.inSwitch() )
-	{
-	  case Dep::PROVIDES_e:    return provides();    break;
-	  case Dep::REQUIRES_e:    return requires();    break;
-	  case Dep::CONFLICTS_e:   return conflicts();   break;
-	  case Dep::OBSOLETES_e:   return obsoletes();   break;
-	  case Dep::RECOMMENDS_e:  return recommends();  break;
-	  case Dep::SUGGESTS_e:    return suggests();    break;
-	  case Dep::ENHANCES_e:    return enhances();    break;
-	  case Dep::SUPPLEMENTS_e: return supplements(); break;
-	  case Dep::PREREQUIRES_e: return prerequires(); break;
-	}
-	return Capabilities();
+        switch( which_r.inSwitch() )
+        {
+          case Dep::PROVIDES_e:    return provides();    break;
+          case Dep::REQUIRES_e:    return requires();    break;
+          case Dep::CONFLICTS_e:   return conflicts();   break;
+          case Dep::OBSOLETES_e:   return obsoletes();   break;
+          case Dep::RECOMMENDS_e:  return recommends();  break;
+          case Dep::SUGGESTS_e:    return suggests();    break;
+          case Dep::ENHANCES_e:    return enhances();    break;
+          case Dep::SUPPLEMENTS_e: return supplements(); break;
+          case Dep::PREREQUIRES_e: return prerequires(); break;
+        }
+        return Capabilities();
       }
       /** \overload operator[] */
       Capabilities operator[]( Dep which_r ) const
@@ -223,6 +261,8 @@ namespace zypp
       CapabilitySet valuesOfNamespace( const std::string & namespace_r ) const;
       //@}
 
+      std::pair<bool, CapabilitySet> matchesSolvable ( const SolvAttr &attr, const sat::Solvable &solv ) const;
+
     public:
       /** \name Locale support. */
       //@{
@@ -304,21 +344,21 @@ namespace zypp
       class SplitIdent
       {
       public:
-	SplitIdent() {}
-	SplitIdent( IdString ident_r );
-	SplitIdent( const char * ident_r );
-	SplitIdent( const std::string & ident_r );
-	SplitIdent( ResKind kind_r, IdString name_r );
-	SplitIdent( ResKind kind_r, const C_Str & name_r );
+        SplitIdent() {}
+        SplitIdent( IdString ident_r );
+        SplitIdent( const char * ident_r );
+        SplitIdent( const std::string & ident_r );
+        SplitIdent( ResKind kind_r, IdString name_r );
+        SplitIdent( ResKind kind_r, const C_Str & name_r );
 
-	IdString ident() const { return _ident; }
-	ResKind  kind()  const { return _kind; }
-	IdString name()  const { return _name; }
+        IdString ident() const { return _ident; }
+        ResKind  kind()  const { return _kind; }
+        IdString name()  const { return _name; }
 
       private:
-	IdString  _ident;
-	ResKind   _kind;
-	IdString  _name;
+        IdString  _ident;
+        ResKind   _kind;
+        IdString  _name;
       };
 
     public:
@@ -433,8 +473,8 @@ namespace zypp
       int res = 0;
       if ( lhs != rhs )
       {
-	if ( (res = lhs.kind().compare( rhs.kind() )) == 0 )
-	  res = lhs.name().compare( rhs.name() );
+        if ( (res = lhs.kind().compare( rhs.kind() )) == 0 )
+          res = lhs.name().compare( rhs.name() );
       }
       return res;
     }
@@ -444,7 +484,7 @@ namespace zypp
     {
       int res = compareByN( lhs, rhs );
       if ( res == 0 )
-	res = lhs.edition().compare( rhs.edition() );
+        res = lhs.edition().compare( rhs.edition() );
       return res;
     }
 
@@ -453,7 +493,7 @@ namespace zypp
     {
       int res = compareByNVR( lhs, rhs );
       if ( res == 0 )
-	res = lhs.arch().compare( rhs.arch() );
+        res = lhs.arch().compare( rhs.arch() );
       return res;
     }
 
