@@ -97,6 +97,9 @@ Resolver::~Resolver()
   delete _satResolver;
 }
 
+sat::detail::CSolver * Resolver::get() const
+{ return _satResolver->get(); }
+
 //---------------------------------------------------------------------------
 // forward flags too SATResolver
 
@@ -187,16 +190,16 @@ void Resolver::removeQueueItem( SolverQueueItem_Ptr item )
 {
     bool found = false;
     for (SolverQueueItemList::const_iterator iter = _added_queue_items.begin();
-	 iter != _added_queue_items.end(); iter++) {
-	if (*iter == item) {
-	    _added_queue_items.remove(*iter);
-	    found = true;
-	    break;
-	}
+         iter != _added_queue_items.end(); iter++) {
+        if (*iter == item) {
+            _added_queue_items.remove(*iter);
+            found = true;
+            break;
+        }
     }
     if (!found) {
-	_removed_queue_items.push_back (item);
-	_removed_queue_items.unique ();
+        _removed_queue_items.push_back (item);
+        _removed_queue_items.unique ();
     }
 }
 
@@ -204,16 +207,16 @@ void Resolver::addQueueItem( SolverQueueItem_Ptr item )
 {
     bool found = false;
     for (SolverQueueItemList::const_iterator iter = _removed_queue_items.begin();
-	 iter != _removed_queue_items.end(); iter++) {
-	if (*iter == item) {
-	    _removed_queue_items.remove(*iter);
-	    found = true;
-	    break;
-	}
+         iter != _removed_queue_items.end(); iter++) {
+        if (*iter == item) {
+            _removed_queue_items.remove(*iter);
+            found = true;
+            break;
+        }
     }
     if (!found) {
-	_added_queue_items.push_back (item);
-	_added_queue_items.unique ();
+        _added_queue_items.push_back (item);
+        _added_queue_items.unique ();
     }
 }
 
@@ -226,13 +229,13 @@ struct UndoTransact : public resfilter::PoolItemFilterFunctor
 {
     ResStatus::TransactByValue resStatus;
     UndoTransact ( const ResStatus::TransactByValue &status)
-	:resStatus(status)
+        :resStatus(status)
     { }
 
     bool operator()( PoolItem item )		// only transacts() items go here
     {
-	item.status().resetTransact( resStatus );// clear any solver/establish transactions
-	return true;
+        item.status().resetTransact( resStatus );// clear any solver/establish transactions
+        return true;
     }
 };
 
@@ -241,13 +244,13 @@ struct DoTransact : public resfilter::PoolItemFilterFunctor
 {
     ResStatus::TransactByValue resStatus;
     DoTransact ( const ResStatus::TransactByValue &status)
-	:resStatus(status)
+        :resStatus(status)
     { }
 
     bool operator()( PoolItem item )		// only transacts() items go here
     {
-	item.status().setTransact( true, resStatus );
-	return true;
+        item.status().setTransact( true, resStatus );
+        return true;
     }
 };
 
@@ -261,8 +264,8 @@ bool Resolver::verifySystem()
     _verifying = true;
 
     invokeOnEach ( _pool.begin(), _pool.end(),
-		   resfilter::ByTransact( ),			// Resetting all transcations
-		   functor::functorRef<bool,PoolItem>(resetting) );
+                   resfilter::ByTransact( ),			// Resetting all transcations
+                   functor::functorRef<bool,PoolItem>(resetting) );
 
     return resolvePool();
 }
@@ -275,8 +278,8 @@ void Resolver::undo()
     UndoTransact info(ResStatus::APPL_LOW);
     MIL << "*** undo ***" << endl;
     invokeOnEach ( _pool.begin(), _pool.end(),
-		   resfilter::ByTransact( ),			// collect transacts from Pool to resolver queue
-		   functor::functorRef<bool,PoolItem>(info) );
+                   resfilter::ByTransact( ),			// collect transacts from Pool to resolver queue
+                   functor::functorRef<bool,PoolItem>(info) );
     //  Regard dependencies of the item weak onl
     _addWeak.clear();
 
@@ -292,14 +295,14 @@ void Resolver::solverInit()
     // Solving with libsolv
     static bool poolDumped = false;
     MIL << "-------------- Calling SAT Solver -------------------" << endl;
-    if ( getenv("ZYPP_FULLLOG") ) {
-	Testcase testcase("/var/log/YaST2/autoTestcase");
-	if (!poolDumped) {
-	    testcase.createTestcase (*this, true, false); // dump pool
-	    poolDumped = true;
-	} else {
-	    testcase.createTestcase (*this, false, false); // write control file only
-	}
+    if ( getenv("ZYPP_FULLLOG") and get() ) { // libzypp/issues/317: get() to make sure a satsolver instance is actually present
+        Testcase testcase("/var/log/YaST2/autoTestcase");
+        if (!poolDumped) {
+            testcase.createTestcase (*this, true, false); // dump pool
+            poolDumped = true;
+        } else {
+            testcase.createTestcase (*this, false, false); // write control file only
+        }
     }
 
     _satResolver->setFixsystem			( isVerifyingMode() );
@@ -334,29 +337,29 @@ bool Resolver::resolveQueue( solver::detail::SolverQueueItemList & queue )
 
     // add/remove additional SolverQueueItems
     for (SolverQueueItemList::const_iterator iter = _removed_queue_items.begin();
-	 iter != _removed_queue_items.end(); iter++) {
-	for (SolverQueueItemList::const_iterator iterQueue = queue.begin(); iterQueue != queue.end(); iterQueue++) {
-	    if ( (*iterQueue)->cmp(*iter) == 0) {
-		MIL << "remove from queue" << *iter;
-		queue.remove(*iterQueue);
-		break;
-	    }
-	}
+         iter != _removed_queue_items.end(); iter++) {
+        for (SolverQueueItemList::const_iterator iterQueue = queue.begin(); iterQueue != queue.end(); iterQueue++) {
+            if ( (*iterQueue)->cmp(*iter) == 0) {
+                MIL << "remove from queue" << *iter;
+                queue.remove(*iterQueue);
+                break;
+            }
+        }
     }
 
     for (SolverQueueItemList::const_iterator iter = _added_queue_items.begin();
-	 iter != _added_queue_items.end(); iter++) {
-	bool found = false;
-	for (SolverQueueItemList::const_iterator iterQueue = queue.begin(); iterQueue != queue.end(); iterQueue++) {
-	    if ( (*iterQueue)->cmp(*iter) == 0) {
-		found = true;
-		break;
-	    }
-	}
-	if (!found) {
-	    MIL << "add to queue" << *iter;
-	    queue.push_back(*iter);
-	}
+         iter != _added_queue_items.end(); iter++) {
+        bool found = false;
+        for (SolverQueueItemList::const_iterator iterQueue = queue.begin(); iterQueue != queue.end(); iterQueue++) {
+            if ( (*iterQueue)->cmp(*iter) == 0) {
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            MIL << "add to queue" << *iter;
+            queue.push_back(*iter);
+        }
     }
 
     // The application has to take care to write these solutions back to e.g. selectables in order
@@ -415,147 +418,147 @@ bool Resolver::applySolution( const ProblemSolution & solution )
 void Resolver::collectResolverInfo()
 {
     if ( _satResolver
-	 && _isInstalledBy.empty()
-	 && _installs.empty()) {
+         && _isInstalledBy.empty()
+         && _installs.empty()) {
 
-	// generating new
-	PoolItemList itemsToInstall = _satResolver->resultItemsToInstall();
+        // generating new
+        PoolItemList itemsToInstall = _satResolver->resultItemsToInstall();
 
-	for (PoolItemList::const_iterator instIter = itemsToInstall.begin();
-	     instIter != itemsToInstall.end(); instIter++) {
-	    // Requires
-	    for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::REQUIRES).begin(); capIt != (*instIter)->dep (Dep::REQUIRES).end(); ++capIt)
-	    {
-		sat::WhatProvides possibleProviders(*capIt);
-		for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
-		    PoolItem provider = ResPool::instance().find( *iter );
+        for (PoolItemList::const_iterator instIter = itemsToInstall.begin();
+             instIter != itemsToInstall.end(); instIter++) {
+            // Requires
+            for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::REQUIRES).begin(); capIt != (*instIter)->dep (Dep::REQUIRES).end(); ++capIt)
+            {
+                sat::WhatProvides possibleProviders(*capIt);
+                for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
+                    PoolItem provider = ResPool::instance().find( *iter );
 
-		    // searching if this provider will already be installed
-		    bool found = false;
-		    bool alreadySetForInstallation = false;
-		    ItemCapKindMap::const_iterator pos = _isInstalledBy.find(provider);
-		    while (pos != _isInstalledBy.end()
-			   && pos->first == provider
-			   && !found) {
-			alreadySetForInstallation = true;
-			ItemCapKind capKind = pos->second;
-			if (capKind.item() == *instIter)  found = true;
-			pos++;
-		    }
+                    // searching if this provider will already be installed
+                    bool found = false;
+                    bool alreadySetForInstallation = false;
+                    ItemCapKindMap::const_iterator pos = _isInstalledBy.find(provider);
+                    while (pos != _isInstalledBy.end()
+                           && pos->first == provider
+                           && !found) {
+                        alreadySetForInstallation = true;
+                        ItemCapKind capKind = pos->second;
+                        if (capKind.item() == *instIter)  found = true;
+                        pos++;
+                    }
 
-		    if (!found
-			&& provider.status().isToBeInstalled()) {
-			if (provider.status().isBySolver()) {
-			    ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::REQUIRES, !alreadySetForInstallation );
-			    _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
-			} else {
-			    // no initial installation cause it has been set be e.g. user
-			    ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::REQUIRES, false );
-			    _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
-			}
-			ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::REQUIRES, !alreadySetForInstallation );
-			_installs.insert (make_pair( *instIter, capKindisInstalledBy));
-		    }
+                    if (!found
+                        && provider.status().isToBeInstalled()) {
+                        if (provider.status().isBySolver()) {
+                            ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::REQUIRES, !alreadySetForInstallation );
+                            _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
+                        } else {
+                            // no initial installation cause it has been set be e.g. user
+                            ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::REQUIRES, false );
+                            _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
+                        }
+                        ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::REQUIRES, !alreadySetForInstallation );
+                        _installs.insert (make_pair( *instIter, capKindisInstalledBy));
+                    }
 
-		    if (provider.status().staysInstalled()) { // Is already satisfied by an item which is installed
-			ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::REQUIRES, false );
-			_satifiedByInstalled.insert (make_pair( *instIter, capKindisInstalledBy));
+                    if (provider.status().staysInstalled()) { // Is already satisfied by an item which is installed
+                        ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::REQUIRES, false );
+                        _satifiedByInstalled.insert (make_pair( *instIter, capKindisInstalledBy));
 
-			ItemCapKind installedSatisfied( *instIter, *capIt, Dep::REQUIRES, false );
-			_installedSatisfied.insert (make_pair( provider, installedSatisfied));
-		    }
-		}
-	    }
+                        ItemCapKind installedSatisfied( *instIter, *capIt, Dep::REQUIRES, false );
+                        _installedSatisfied.insert (make_pair( provider, installedSatisfied));
+                    }
+                }
+            }
 
-	    if (!(_satResolver->onlyRequires())) {
-		//Recommends
-		for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::RECOMMENDS).begin(); capIt != (*instIter)->dep (Dep::RECOMMENDS).end(); ++capIt)
-		{
-		    sat::WhatProvides possibleProviders(*capIt);
-		    for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
-			PoolItem provider = ResPool::instance().find( *iter );
+            if (!(_satResolver->onlyRequires())) {
+                //Recommends
+                for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::RECOMMENDS).begin(); capIt != (*instIter)->dep (Dep::RECOMMENDS).end(); ++capIt)
+                {
+                    sat::WhatProvides possibleProviders(*capIt);
+                    for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
+                        PoolItem provider = ResPool::instance().find( *iter );
 
-			// searching if this provider will already be installed
-			bool found = false;
-			bool alreadySetForInstallation = false;
-			ItemCapKindMap::const_iterator pos = _isInstalledBy.find(provider);
-			while (pos != _isInstalledBy.end()
-			       && pos->first == provider
-			       && !found) {
-			    alreadySetForInstallation = true;
-			    ItemCapKind capKind = pos->second;
-			    if (capKind.item() == *instIter)  found = true;
-			    pos++;
-			}
+                        // searching if this provider will already be installed
+                        bool found = false;
+                        bool alreadySetForInstallation = false;
+                        ItemCapKindMap::const_iterator pos = _isInstalledBy.find(provider);
+                        while (pos != _isInstalledBy.end()
+                               && pos->first == provider
+                               && !found) {
+                            alreadySetForInstallation = true;
+                            ItemCapKind capKind = pos->second;
+                            if (capKind.item() == *instIter)  found = true;
+                            pos++;
+                        }
 
-			if (!found
-			    && provider.status().isToBeInstalled()) {
-			    if (provider.status().isBySolver()) {
-				ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::RECOMMENDS, !alreadySetForInstallation );
-				_isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
-			    } else {
-				// no initial installation cause it has been set be e.g. user
-				ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::RECOMMENDS, false );
-				_isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
-			    }
-			    ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::RECOMMENDS, !alreadySetForInstallation );
-			    _installs.insert (make_pair( *instIter, capKindisInstalledBy));
-			}
+                        if (!found
+                            && provider.status().isToBeInstalled()) {
+                            if (provider.status().isBySolver()) {
+                                ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::RECOMMENDS, !alreadySetForInstallation );
+                                _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
+                            } else {
+                                // no initial installation cause it has been set be e.g. user
+                                ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::RECOMMENDS, false );
+                                _isInstalledBy.insert (make_pair( provider, capKindisInstalledBy));
+                            }
+                            ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::RECOMMENDS, !alreadySetForInstallation );
+                            _installs.insert (make_pair( *instIter, capKindisInstalledBy));
+                        }
 
-			if (provider.status().staysInstalled()) { // Is already satisfied by an item which is installed
-			    ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::RECOMMENDS, false );
-			    _satifiedByInstalled.insert (make_pair( *instIter, capKindisInstalledBy));
+                        if (provider.status().staysInstalled()) { // Is already satisfied by an item which is installed
+                            ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::RECOMMENDS, false );
+                            _satifiedByInstalled.insert (make_pair( *instIter, capKindisInstalledBy));
 
-			    ItemCapKind installedSatisfied( *instIter, *capIt, Dep::RECOMMENDS, false );
-			    _installedSatisfied.insert (make_pair( provider, installedSatisfied));
-			}
-		    }
-		}
+                            ItemCapKind installedSatisfied( *instIter, *capIt, Dep::RECOMMENDS, false );
+                            _installedSatisfied.insert (make_pair( provider, installedSatisfied));
+                        }
+                    }
+                }
 
-		//Supplements
-		for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::SUPPLEMENTS).begin(); capIt != (*instIter)->dep (Dep::SUPPLEMENTS).end(); ++capIt)
-		{
-		    sat::WhatProvides possibleProviders(*capIt);
-		    for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
-			PoolItem provider = ResPool::instance().find( *iter );
-			// searching if this item will already be installed
-			bool found = false;
-			bool alreadySetForInstallation = false;
-			ItemCapKindMap::const_iterator pos = _isInstalledBy.find(*instIter);
-			while (pos != _isInstalledBy.end()
-			       && pos->first == *instIter
-			       && !found) {
-			    alreadySetForInstallation = true;
-			    ItemCapKind capKind = pos->second;
-			    if (capKind.item() == provider)  found = true;
-			    pos++;
-			}
+                //Supplements
+                for (Capabilities::const_iterator capIt = (*instIter)->dep (Dep::SUPPLEMENTS).begin(); capIt != (*instIter)->dep (Dep::SUPPLEMENTS).end(); ++capIt)
+                {
+                    sat::WhatProvides possibleProviders(*capIt);
+                    for_( iter, possibleProviders.begin(), possibleProviders.end() ) {
+                        PoolItem provider = ResPool::instance().find( *iter );
+                        // searching if this item will already be installed
+                        bool found = false;
+                        bool alreadySetForInstallation = false;
+                        ItemCapKindMap::const_iterator pos = _isInstalledBy.find(*instIter);
+                        while (pos != _isInstalledBy.end()
+                               && pos->first == *instIter
+                               && !found) {
+                            alreadySetForInstallation = true;
+                            ItemCapKind capKind = pos->second;
+                            if (capKind.item() == provider)  found = true;
+                            pos++;
+                        }
 
-			if (!found
-			    && instIter->status().isToBeInstalled()) {
-			    if (instIter->status().isBySolver()) {
-				ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
-				_isInstalledBy.insert (make_pair( *instIter, capKindisInstalledBy));
-			    } else {
-				// no initial installation cause it has been set be e.g. user
-				ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::SUPPLEMENTS, false );
-				_isInstalledBy.insert (make_pair( *instIter, capKindisInstalledBy));
-			    }
-			    ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
-			    _installs.insert (make_pair( provider, capKindisInstalledBy));
-			}
+                        if (!found
+                            && instIter->status().isToBeInstalled()) {
+                            if (instIter->status().isBySolver()) {
+                                ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
+                                _isInstalledBy.insert (make_pair( *instIter, capKindisInstalledBy));
+                            } else {
+                                // no initial installation cause it has been set be e.g. user
+                                ItemCapKind capKindisInstalledBy( provider, *capIt, Dep::SUPPLEMENTS, false );
+                                _isInstalledBy.insert (make_pair( *instIter, capKindisInstalledBy));
+                            }
+                            ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
+                            _installs.insert (make_pair( provider, capKindisInstalledBy));
+                        }
 
-			if (instIter->status().staysInstalled()) { // Is already satisfied by an item which is installed
-			    ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
-			    _satifiedByInstalled.insert (make_pair( provider, capKindisInstalledBy));
+                        if (instIter->status().staysInstalled()) { // Is already satisfied by an item which is installed
+                            ItemCapKind capKindisInstalledBy( *instIter, *capIt, Dep::SUPPLEMENTS, !alreadySetForInstallation );
+                            _satifiedByInstalled.insert (make_pair( provider, capKindisInstalledBy));
 
-			    ItemCapKind installedSatisfied( provider, *capIt, Dep::SUPPLEMENTS, false );
-			    _installedSatisfied.insert (make_pair( *instIter, installedSatisfied));
-			}
-		    }
-		}
-   	    }
-	}
+                            ItemCapKind installedSatisfied( provider, *capIt, Dep::SUPPLEMENTS, false );
+                            _installedSatisfied.insert (make_pair( *instIter, installedSatisfied));
+                        }
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -566,15 +569,15 @@ ItemCapKindList Resolver::isInstalledBy( const PoolItem & item )
     collectResolverInfo();
 
     for (ItemCapKindMap::const_iterator iter = _isInstalledBy.find(item); iter != _isInstalledBy.end();) {
-	ItemCapKind info = iter->second;
-	PoolItem iterItem = iter->first;
-	if (iterItem == item) {
-	    ret.push_back(info);
-	    iter++;
-	} else {
-	    // exit
-	    iter = _isInstalledBy.end();
-	}
+        ItemCapKind info = iter->second;
+        PoolItem iterItem = iter->first;
+        if (iterItem == item) {
+            ret.push_back(info);
+            iter++;
+        } else {
+            // exit
+            iter = _isInstalledBy.end();
+        }
     }
     return ret;
 }
@@ -585,15 +588,15 @@ ItemCapKindList Resolver::installs( const PoolItem & item )
     collectResolverInfo();
 
     for (ItemCapKindMap::const_iterator iter = _installs.find(item); iter != _installs.end();) {
-	ItemCapKind info = iter->second;
-	PoolItem iterItem = iter->first;
-	if (iterItem == item) {
-	    ret.push_back(info);
-	    iter++;
-	} else {
-	    // exit
-	    iter = _installs.end();
-	}
+        ItemCapKind info = iter->second;
+        PoolItem iterItem = iter->first;
+        if (iterItem == item) {
+            ret.push_back(info);
+            iter++;
+        } else {
+            // exit
+            iter = _installs.end();
+        }
     }
     return ret;
 }
@@ -604,15 +607,15 @@ ItemCapKindList Resolver::satifiedByInstalled( const PoolItem & item )
     collectResolverInfo();
 
     for (ItemCapKindMap::const_iterator iter = _satifiedByInstalled.find(item); iter != _satifiedByInstalled.end();) {
-	ItemCapKind info = iter->second;
-	PoolItem iterItem = iter->first;
-	if (iterItem == item) {
-	    ret.push_back(info);
-	    iter++;
-	} else {
-	    // exit
-	    iter = _satifiedByInstalled.end();
-	}
+        ItemCapKind info = iter->second;
+        PoolItem iterItem = iter->first;
+        if (iterItem == item) {
+            ret.push_back(info);
+            iter++;
+        } else {
+            // exit
+            iter = _satifiedByInstalled.end();
+        }
     }
     return ret;
 }
@@ -623,15 +626,15 @@ ItemCapKindList Resolver::installedSatisfied( const PoolItem & item )
     collectResolverInfo();
 
     for (ItemCapKindMap::const_iterator iter = _installedSatisfied.find(item); iter != _installedSatisfied.end();) {
-	ItemCapKind info = iter->second;
-	PoolItem iterItem = iter->first;
-	if (iterItem == item) {
-	    ret.push_back(info);
-	    iter++;
-	} else {
-	    // exit
-	    iter = _installedSatisfied.end();
-	}
+        ItemCapKind info = iter->second;
+        PoolItem iterItem = iter->first;
+        if (iterItem == item) {
+            ret.push_back(info);
+            iter++;
+        } else {
+            // exit
+            iter = _installedSatisfied.end();
+        }
     }
     return ret;
 }
