@@ -29,17 +29,18 @@ extern "C"
 #include <vector>
 #include <algorithm>
 
+#include <zypp-core/base/StringV.h>
 #include <zypp/base/Logger.h>
 #include <zypp/base/String.h>
 #include <zypp/base/Gettext.h>
 #include <zypp/base/LocaleGuard.h>
-#include <zypp/base/DtorReset.h>
+#include <zypp-core/base/DtorReset>
 
 #include <zypp/Date.h>
 #include <zypp/Pathname.h>
 #include <zypp/PathInfo.h>
 #include <zypp/PublicKey.h>
-#include <zypp/ProgressData.h>
+#include <zypp-core/ui/ProgressData>
 
 #include <zypp/target/rpm/RpmDb.h>
 #include <zypp/target/rpm/RpmCallbacks.h>
@@ -49,6 +50,7 @@ extern "C"
 #include <zypp/target/rpm/RpmException.h>
 #include <zypp/TmpPath.h>
 #include <zypp/KeyRing.h>
+#include <zypp/KeyManager.h>
 #include <zypp/ZYppFactory.h>
 #include <zypp/ZConfig.h>
 #include <zypp/base/IOTools.h>
@@ -76,8 +78,8 @@ namespace zypp
     inline bool ZYPP_RPM_DEBUG()
     {
       static bool val = [](){
-	const char * env = getenv("ZYPP_RPM_DEBUG");
-	return( env && str::strToBool( env, true ) );
+        const char * env = getenv("ZYPP_RPM_DEBUG");
+        return( env && str::strToBool( env, true ) );
       }();
       return val;
     }
@@ -122,7 +124,7 @@ inline std::string rpmQuoteFilename( const Pathname & path_r )
       // try to prepend cwd
       AutoDispose<char*> cwd( ::get_current_dir_name(), ::free );
       if ( cwd )
-	return Pathname( cwd ) / path_r;
+        return Pathname( cwd ) / path_r;
       WAR << "Can't get cwd!" << endl;
     }
 #endif
@@ -271,44 +273,39 @@ void RpmDb::initDatabase( Pathname root_r, bool doRebuild_r )
   if ( root_r.empty() )
     root_r = "/";
 
-  // NOTE: Former argument, but now locked to "/var/lib/rpm".
-  // A custom dbPath is not actually needed and would only work
-  // reliably if libsolv also supports it. By now no further
-  // cleanup in the code.
-  const Pathname & dbPath_r { librpmDb::defaultDbPath() };
+  const Pathname & dbPath_r { librpmDb::suggestedDbPath( root_r ) };	// also asserts root_r is absolute
 
-  if ( ! root_r.absolute() )
+  // The rpmdb compat symlink.
+  // Required at least until rpmdb2solv takes a dppath argument.
+  // Otherwise it creates a db at "/var/lib/rpm".
+  if ( dbPath_r != "/var/lib/rpm" && ! PathInfo( root_r/"/var/lib/rpm" ).isExist() )
   {
-    ERR << "Illegal root or dbPath: " << stringPath( root_r, dbPath_r ) << endl;
-    ZYPP_THROW(RpmInvalidRootException(root_r, dbPath_r));
+    WAR << "Inject missing /var/lib/rpm compat symlink to " << dbPath_r << endl;
+    filesystem::assert_dir( root_r/"/var/lib" );
+    filesystem::symlink( "../../"/dbPath_r, root_r/"/var/lib/rpm" );
   }
 
-  if ( ! PathInfo( root_r/"/var/lib/rpm" ).isExist()
-    && PathInfo( root_r/"/usr/lib/sysimage/rpm" ).isDir() )
-  {
-    WAR << "Rpm package was deleted? Injecting missing rpmdb compat symlink." << endl;
-    filesystem::symlink( "../../usr/lib/sysimage/rpm", root_r/"/var/lib/rpm" );
-  }
-
-  MIL << "Calling initDatabase: " << stringPath( root_r, dbPath_r )
-      << ( doRebuild_r ? " (rebuilddb)" : "" )
-      << ( quickinit ? " (quickinit)" : "" ) << endl;
-
   ///////////////////////////////////////////////////////////////////
   // Check whether already initialized
   ///////////////////////////////////////////////////////////////////
   if ( initialized() )
   {
-    if ( root_r == _root && dbPath_r == _dbPath )
-    {
+    // Just check for a changing root because the librpmDb::suggestedDbPath
+    // may indeed change: rpm %post moving the db from /var/lib/rpm
+    // to /usr/lib/sysimage/rpm. We continue to use the old dbpath
+    // (via the compat symlink) until a re-init.
+    if ( root_r == _root ) {
+      MIL << "Calling initDatabase: already initialized at " << stringPath( _root, _dbPath ) << endl;
       return;
     }
     else
-    {
       ZYPP_THROW(RpmDbAlreadyOpenException(_root, _dbPath, root_r, dbPath_r));
-    }
   }
 
+  MIL << "Calling initDatabase: " << stringPath( root_r, dbPath_r )
+      << ( doRebuild_r ? " (rebuilddb)" : "" )
+      << ( quickinit ? " (quickinit)" : "" ) << endl;
+
   ///////////////////////////////////////////////////////////////////
   // init database
   ///////////////////////////////////////////////////////////////////
@@ -449,17 +446,17 @@ void RpmDb::doRebuildDatabase(callback::SendReport<RebuildDBReport> & report)
     {
       if ( ! str::endsWith( line, ignoreSuffix ) )
       {
-	errmsg += line;
-	errmsg += '\n';
-	WAR << line << endl;
+        errmsg += line;
+        errmsg += '\n';
+        WAR << line << endl;
       }
     }
     else if ( str::startsWith( line, progressPrefix ) )
     {
       if ( ! tics.incr() )
       {
-	WAR << "User requested abort." << endl;
-	systemKill();
+        WAR << "User requested abort." << endl;
+        systemKill();
       }
     }
   }
@@ -489,56 +486,56 @@ namespace
     struct Key
     {
       Key()
-	: _inRpmKeys( nullptr )
-	, _inZyppKeys( nullptr )
+        : _inRpmKeys( nullptr )
+        , _inZyppKeys( nullptr )
       {}
 
       void updateIf( const Edition & rpmKey_r )
       {
-	std::string keyRelease( rpmKey_r.release() );
-	int comp = _release.compare( keyRelease );
-	if ( comp < 0 )
-	{
-	  // update to newer release
-	  _release.swap( keyRelease );
-	  _inRpmKeys  = &rpmKey_r;
-	  _inZyppKeys = nullptr;
-	  if ( !keyRelease.empty() )
-	    DBG << "Old key in Z: gpg-pubkey-" << rpmKey_r.version() << "-" <<  keyRelease << endl;
-	}
-	else if ( comp == 0 )
-	{
-	  // stay with this release
-	  if ( ! _inRpmKeys )
-	    _inRpmKeys = &rpmKey_r;
-	}
-	// else: this is an old release
-	else
-	  DBG << "Old key in R: gpg-pubkey-" << rpmKey_r.version() << "-" <<  keyRelease << endl;
+        std::string keyRelease( rpmKey_r.release() );
+        int comp = _release.compare( keyRelease );
+        if ( comp < 0 )
+        {
+          // update to newer release
+          _release.swap( keyRelease );
+          _inRpmKeys  = &rpmKey_r;
+          _inZyppKeys = nullptr;
+          if ( !keyRelease.empty() )
+            DBG << "Old key in Z: gpg-pubkey-" << rpmKey_r.version() << "-" <<  keyRelease << endl;
+        }
+        else if ( comp == 0 )
+        {
+          // stay with this release
+          if ( ! _inRpmKeys )
+            _inRpmKeys = &rpmKey_r;
+        }
+        // else: this is an old release
+        else
+          DBG << "Old key in R: gpg-pubkey-" << rpmKey_r.version() << "-" <<  keyRelease << endl;
       }
 
       void updateIf( const PublicKeyData & zyppKey_r )
       {
-	std::string keyRelease( zyppKey_r.gpgPubkeyRelease() );
-	int comp = _release.compare( keyRelease );
-	if ( comp < 0 )
-	{
-	  // update to newer release
-	  _release.swap( keyRelease );
-	  _inRpmKeys  = nullptr;
-	  _inZyppKeys = &zyppKey_r;
-	  if ( !keyRelease.empty() )
-	    DBG << "Old key in R: gpg-pubkey-" << zyppKey_r.gpgPubkeyVersion() << "-" << keyRelease << endl;
-	}
-	else if ( comp == 0 )
-	{
-	  // stay with this release
-	  if ( ! _inZyppKeys )
-	    _inZyppKeys = &zyppKey_r;
-	}
-	// else: this is an old release
-	else
-	  DBG << "Old key in Z: gpg-pubkey-" << zyppKey_r.gpgPubkeyVersion() << "-" << keyRelease << endl;
+        std::string keyRelease( zyppKey_r.gpgPubkeyRelease() );
+        int comp = _release.compare( keyRelease );
+        if ( comp < 0 )
+        {
+          // update to newer release
+          _release.swap( keyRelease );
+          _inRpmKeys  = nullptr;
+          _inZyppKeys = &zyppKey_r;
+          if ( !keyRelease.empty() )
+            DBG << "Old key in R: gpg-pubkey-" << zyppKey_r.gpgPubkeyVersion() << "-" << keyRelease << endl;
+        }
+        else if ( comp == 0 )
+        {
+          // stay with this release
+          if ( ! _inZyppKeys )
+            _inZyppKeys = &zyppKey_r;
+        }
+        // else: this is an old release
+        else
+          DBG << "Old key in Z: gpg-pubkey-" << zyppKey_r.gpgPubkeyVersion() << "-" << keyRelease << endl;
       }
 
       std::string _release;
@@ -567,14 +564,14 @@ namespace
     {
       DBG << "gpg-pubkey-" << (*it).first << "-" << (*it).second._release << " "
           << ( (*it).second._inRpmKeys  ? "R" : "_" )
-	  << ( (*it).second._inZyppKeys ? "Z" : "_" ) << endl;
+          << ( (*it).second._inZyppKeys ? "Z" : "_" ) << endl;
       if ( ! (*it).second._inRpmKeys )
       {
-	zyppKeys.push_back( *(*it).second._inZyppKeys );
+        zyppKeys.push_back( *(*it).second._inZyppKeys );
       }
       if ( ! (*it).second._inZyppKeys )
       {
-	rpmKeys.insert( *(*it).second._inRpmKeys );
+        rpmKeys.insert( *(*it).second._inRpmKeys );
       }
     }
     rpmKeys_r.swap( rpmKeys );
@@ -607,9 +604,9 @@ void RpmDb::syncTrustedKeys( SyncTrustedKeyBits mode_r )
     {
       if ( ! rpmKeys.count( keyData.gpgPubkeyEdition() ) )
       {
-	DBG << "Excess key in Z to delete: gpg-pubkey-" << keyData.gpgPubkeyEdition() << endl;
-	getZYpp()->keyRing()->deleteKey( keyData.id(), /*trusted*/true );
-	if ( !dirty ) dirty = true;
+        DBG << "Excess key in Z to delete: gpg-pubkey-" << keyData.gpgPubkeyEdition() << endl;
+        getZYpp()->keyRing()->deleteKey( keyData.id(), /*trusted*/true );
+        if ( !dirty ) dirty = true;
       }
     }
     if ( dirty )
@@ -634,10 +631,10 @@ void RpmDb::syncTrustedKeys( SyncTrustedKeyBits mode_r )
       std::ofstream tmpos( tmpfile.path().c_str() );
       for_( it, rpmKeys.begin(), rpmKeys.end() )
       {
-	// we export the rpm key into a file
-	RpmHeader::constPtr result;
-	getData( "gpg-pubkey", *it, result );
-	tmpos << result->tag_description() << endl;
+        // we export the rpm key into a file
+        RpmHeader::constPtr result;
+        getData( "gpg-pubkey", *it, result );
+        tmpos << result->tag_description() << endl;
       }
     }
     try
@@ -649,10 +646,10 @@ void RpmDb::syncTrustedKeys( SyncTrustedKeyBits mode_r )
       std::set<Edition> missingKeys;
       for ( const Edition & key : rpmKeys )
       {
-	if ( getZYpp()->keyRing()->isKeyTrusted( key.version() ) ) // key.version is the gpgkeys short ID
-	  continue;
-	ERR << "Could not import key:" << str::Format("gpg-pubkey-%s") % key << " into zypp keyring (V3 key?)" << endl;
-	missingKeys.insert( key );
+        if ( getZYpp()->keyRing()->isKeyTrusted( key.version() ) ) // key.version is the gpgkeys short ID
+          continue;
+        ERR << "Could not import key:" << str::Format("gpg-pubkey-%s") % key << " into zypp keyring (V3 key?)" << endl;
+        missingKeys.insert( key );
       }
       if ( ! missingKeys.empty() )
         callback::SendReport<KeyRingReport>()->reportNonImportedKeys(missingKeys);
@@ -673,11 +670,11 @@ void RpmDb::syncTrustedKeys( SyncTrustedKeyBits mode_r )
     {
       try
       {
-	importPubkey( getZYpp()->keyRing()->exportTrustedPublicKey( *it ) );
+        importPubkey( getZYpp()->keyRing()->exportTrustedPublicKey( *it ) );
       }
       catch ( const RpmException & exp )
       {
-	ZYPP_CAUGHT( exp );
+        ZYPP_CAUGHT( exp );
       }
     }
   }
@@ -823,8 +820,8 @@ void RpmDb::removePubkey( const PublicKey & pubkey_r )
   {
     if ( (*it).version() == pubkeyVersion )
     {
-	found_edition = it;
-	break;
+        found_edition = it;
+        break;
     }
   }
 
@@ -1114,48 +1111,36 @@ namespace
 {
   struct RpmlogCapture : public std::string
   {
-    RpmlogCapture()
-    { rpmlog()._cap = this; }
+    RpmlogCapture() : _cap(this)
+    {
+      rpmlogSetCallback( rpmLogCB, this );
+      _oldMask = rpmlogSetMask( RPMLOG_UPTO( RPMLOG_PRI(RPMLOG_INFO) ) );
+    }
 
     ~RpmlogCapture()
-    { rpmlog()._cap = nullptr; }
+    {
+      rpmlogSetCallback( nullptr, nullptr );
+      rpmlogSetMask( _oldMask );
+    }
+
+    static int rpmLogCB( rpmlogRec rec_r, rpmlogCallbackData data_r )
+    { return reinterpret_cast<RpmlogCapture*>(data_r)->rpmLog( rec_r ); }
+
+    int rpmLog( rpmlogRec rec_r )
+    {
+      if ( _cap ) (*_cap) += rpmlogRecMessage( rec_r );
+      return 0;
+    }
 
   private:
-    struct Rpmlog
-    {
-      Rpmlog()
-      : _cap( nullptr )
-      {
-	rpmlogSetCallback( rpmLogCB, this );
-	rpmSetVerbosity( RPMLOG_INFO );
-	_f = ::fopen( "/dev/null","w");
-	rpmlogSetFile( _f );
-      }
-
-      ~Rpmlog()
-      { if ( _f ) ::fclose( _f ); }
-
-      static int rpmLogCB( rpmlogRec rec_r, rpmlogCallbackData data_r )
-      { return reinterpret_cast<Rpmlog*>(data_r)->rpmLog( rec_r ); }
-
-      int rpmLog( rpmlogRec rec_r )
-      {
-	if ( _cap ) (*_cap) += rpmlogRecMessage( rec_r );
-	return RPMLOG_DEFAULT;
-      }
-
-      FILE * _f;
-      std::string * _cap;
-    };
-
-    static Rpmlog & rpmlog()
-    { static Rpmlog _rpmlog; return _rpmlog; }
+    std::string * _cap;
+    int _oldMask = 0;
   };
 
   RpmDb::CheckPackageResult doCheckPackageSig( const Pathname & path_r,			// rpm file to check
-					       const Pathname & root_r,			// target root
-					       bool  requireGPGSig_r,			// whether no gpg signature is to be reported
-					       RpmDb::CheckPackageDetail & detail_r )	// detailed result
+                                               const Pathname & root_r,			// target root
+                                               bool  requireGPGSig_r,			// whether no gpg signature is to be reported
+                                               RpmDb::CheckPackageDetail & detail_r )	// detailed result
   {
     PathInfo file( path_r );
     if ( ! file.isFile() )
@@ -1169,7 +1154,7 @@ namespace
     {
       ERR << "Can't open file for reading: " << file << " (" << ::Fstrerror(fd) << ")" << endl;
       if ( fd )
-	::Fclose( fd );
+        ::Fclose( fd );
       return RpmDb::CHK_ERROR;
     }
     rpmts ts = ::rpmtsCreate();
@@ -1178,7 +1163,7 @@ namespace
 
     rpmQVKArguments_s qva;
     memset( &qva, 0, sizeof(rpmQVKArguments_s) );
-#ifdef HAVE_NO_RPMTSSETVFYFLAGS
+#ifndef HAVE_RPM_VERIFY_TRANSACTION_STEP
     // Legacy: In rpm >= 4.15 qva_flags symbols don't exist
     // and qva_flags is not used in signature checking at all.
     qva.qva_flags = (VERIFY_DIGEST|VERIFY_SIGNATURE);
@@ -1199,20 +1184,20 @@ namespace
     //     V3 RSA/SHA256 Signature, key ID 3dbdc284: OK
     //     MD5 digest: OK (fd5259fe677a406951dcb2e9d08c4dcc)
     //
-    // TODO: try to get SIG info from the header rather than parsing the output
     std::vector<std::string> lines;
     str::split( vresult, std::back_inserter(lines), "\n" );
     unsigned count[7] = { 0, 0, 0, 0, 0, 0, 0 };
 
     for ( unsigned i = 1; i < lines.size(); ++i )
     {
-      std::string & line( lines[i] );
+      std::string &line( lines[i] );
+
       RpmDb::CheckPackageResult lineres = RpmDb::CHK_ERROR;
       if ( line.find( ": OK" ) != std::string::npos )
       {
-	lineres = RpmDb::CHK_OK;
-	if ( line.find( "Signature, key ID" ) == std::string::npos )
-	  ++count[RpmDb::CHK_NOSIG];	// Valid but no gpg signature -> CHK_NOSIG
+        lineres = RpmDb::CHK_OK;
+        if ( line.find( "Signature, key ID" ) == std::string::npos )
+          ++count[RpmDb::CHK_NOSIG];	// Valid but no gpg signature -> CHK_NOSIG
       }
       else if ( line.find( ": NOKEY" ) != std::string::npos )
       { lineres = RpmDb::CHK_NOKEY; }
@@ -1247,17 +1232,72 @@ namespace
     {
       if ( count[RpmDb::CHK_OK] == count[RpmDb::CHK_NOSIG]  )
       {
-	detail_r.push_back( RpmDb::CheckPackageDetail::value_type( RpmDb::CHK_NOSIG, std::string("    ")+_("Package is not signed!") ) );
-	if ( requireGPGSig_r )
-	  ret = RpmDb::CHK_NOSIG;
+        detail_r.push_back( RpmDb::CheckPackageDetail::value_type( RpmDb::CHK_NOSIG, std::string("    ")+_("Package is not signed!") ) );
+        if ( requireGPGSig_r )
+          ret = RpmDb::CHK_NOSIG;
       }
     }
 
     if ( ret != RpmDb::CHK_OK )
     {
+
+      bool didReadHeader = false;
+      std::unordered_map< std::string, std::string> fprs;
+
+      // we replace the data only if the key IDs are actually only 8 bytes
+      str::regex rxexpr( "key ID ([a-fA-F0-9]{8}):" );
+      for ( auto &detail : detail_r ) {
+        auto &line = detail.second;
+        str::smatch what;
+        if ( str::regex_match( line, what, rxexpr ) ) {
+
+          if ( !didReadHeader ) {
+            didReadHeader = true;
+
+            // Get signature info from the package header, RPM always prints only the 8 byte ID
+            auto header = RpmHeader::readPackage( path_r, RpmHeader::NOVERIFY );
+            if ( header ) {
+              auto keyMgr = zypp::KeyManagerCtx::createForOpenPGP();
+              const auto &addFprs = [&]( auto tag ){
+                const auto &list1 = keyMgr.readSignatureFingerprints( header->blob_val( tag ) );
+                for ( const auto &id : list1 ) {
+                  if ( id.size() <= 8 )
+                    continue;
+
+                  const auto &lowerId = str::toLower( id );
+                  fprs.insert( std::make_pair( lowerId.substr( lowerId.size() - 8 ), lowerId ) );
+                }
+              };
+
+              addFprs( RPMTAG_SIGGPG );
+              addFprs( RPMTAG_SIGPGP );
+              addFprs( RPMTAG_RSAHEADER );
+              addFprs( RPMTAG_DSAHEADER );
+
+            } else {
+              ERR << "Failed to read package signatures." << std::endl;
+            }
+          }
+
+          // if we have no keys we can substiture we can leave the loop right away
+          if ( !fprs.size() )
+            break;
+
+          {
+            // replace the short key ID with the long ones parsed from the header
+            const auto &keyId = str::toLower( what[1] );
+            if ( const auto &i = fprs.find( keyId ); i != fprs.end() ) {
+              str::replaceAll( line, keyId, i->second );
+            }
+          }
+        }
+      }
+
       WAR << path_r << " (" << requireGPGSig_r << " -> " << ret << ")" << endl;
       WAR << vresult;
     }
+    else
+      DBG << path_r << " [0-Signature is OK]" << endl;
     return ret;
   }
 
@@ -1335,7 +1375,6 @@ RpmDb::queryChangedFiles(FileList & fileList, const std::string& packageName)
 }
 
 
-
 /****************************************************************/
 /* private member-functions					*/
 /****************************************************************/
@@ -1608,7 +1647,6 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
 
   MIL << "RpmDb::installPackage(" << filename << "," << flags << ")" << endl;
 
-
   // backup
   if ( _packagebackups )
   {
@@ -1672,7 +1710,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
   cmdout.set( "lineno", lineno );
 
   // LEGACY: collect and forward additional rpm output in finish
-  std::string rpmmsg;				// TODO: immediately forward lines via Callback::report rather than collecting
+  std::string rpmmsg;
   std::vector<std::string> configwarnings;	// TODO: immediately process lines rather than collecting
 
   while ( systemReadLine( line ) )
@@ -1690,7 +1728,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
 
     if ( lineno >= MAXRPMMESSAGELINES ) {
       if ( line.find( " scriptlet failed, " ) == std::string::npos )	// always log %script errors
-	continue;
+        continue;
     }
 
     rpmmsg += line+'\n';
@@ -1728,7 +1766,10 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
     sstr << "rpm output:" << endl << rpmmsg << endl;
     historylog.comment(sstr.str());
     // TranslatorExplanation the colon is followed by an error message
-    ZYPP_THROW(RpmSubprocessException(_("RPM failed: ") + (rpmmsg.empty() ? error_message : rpmmsg) ));
+    auto excpt { RpmSubprocessException(_("RPM failed: ") + error_message ) };
+    if ( not rpmmsg.empty() )
+      excpt.addHistory( rpmmsg );
+    ZYPP_THROW(std::move(excpt));
   }
   else if ( ! rpmmsg.empty() )
   {
@@ -1739,7 +1780,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, cal
     sstr << "Additional rpm output:" << endl << rpmmsg << endl;
     historylog.comment(sstr.str());
 
-    // report additional rpm output in finish
+    // report additional rpm output in finish (LEGACY! Lines are immediately reported as InstallResolvableReport::contentRpmout)
     // TranslatorExplanation Text is followed by a ':'  and the actual output.
     report->finishInfo(str::form( "%s:\n%s\n", _("Additional rpm output"),  rpmmsg.c_str() ));
   }
@@ -1852,7 +1893,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
 
 
   // LEGACY: collect and forward additional rpm output in finish
-  std::string rpmmsg;		// TODO: immediately forward lines via Callback::report rather than collecting
+  std::string rpmmsg;
 
   // got no progress from command, so we fake it:
   // 5  - command started
@@ -1867,7 +1908,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
 
     if ( lineno >= MAXRPMMESSAGELINES ) {
       if ( line.find( " scriptlet failed, " ) == std::string::npos )	// always log %script errors
-	continue;
+        continue;
     }
     rpmmsg += line+'\n';
   }
@@ -1884,7 +1925,10 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
     sstr << "rpm output:" << endl << rpmmsg << endl;
     historylog.comment(sstr.str());
     // TranslatorExplanation the colon is followed by an error message
-    ZYPP_THROW(RpmSubprocessException(_("RPM failed: ") + (rpmmsg.empty() ? error_message: rpmmsg) ));
+    auto excpt { RpmSubprocessException(_("RPM failed: ") + error_message ) };
+    if ( not rpmmsg.empty() )
+      excpt.addHistory( rpmmsg );
+    ZYPP_THROW(std::move(excpt));
   }
   else if ( ! rpmmsg.empty() )
   {
@@ -1895,7 +1939,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, cal
     sstr << "Additional rpm output:" << endl << rpmmsg << endl;
     historylog.comment(sstr.str());
 
-    // report additional rpm output in finish
+    // report additional rpm output in finish (LEGACY! Lines are immediately reported as RemoveResolvableReport::contentRpmout)
     // TranslatorExplanation Text is followed by a ':'  and the actual output.
     report->finishInfo(str::form( "%s:\n%s\n", _("Additional rpm output"),  rpmmsg.c_str() ));
   }
